"""
dolbyio_rest_apis.streaming.stream
~~~~~~~~~~~~~~~

This module contains the functions to work with the Stream APIs.
"""

from dolbyio_rest_apis.core.urls import get_rts_url
from dolbyio_rest_apis.streaming.internal.http_context import StreamingHttpContext

async def stop(
        api_secret: str,
        account_id: str,
        stream_name: str,
    ) -> None:
    payload = {
        'streamId': f'{account_id}/{stream_name}',
    }

    async with StreamingHttpContext() as http_context:
        await http_context.requests_post(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/stream/stop',
            payload=payload,
        )

async def stop_all(
        api_secret: str,
    ) -> None:
    async with StreamingHttpContext() as http_context:
        await http_context.requests_post(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/stream/stop/all',
        )



"""
dolbyio_rest_apis.communications.remix
~~~~~~~~~~~~~~~

This module contains the functions to work with the remix API.
"""

from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.core.urls import get_comms_url_v2
from .models import RemixStatus

async def start(
        access_token: str,
        conference_id: str,
        layout_url: str=None,
        layout_name: str=None,
    ) -> RemixStatus:
    r"""
    Remix a conference.

    Use this API to trigger a remix and regenerate a recording of a previously recorded conference using
    the current mixer layout. The `Recording.MP4.Available` event is sent if the customer has configured
    the webhook in the developer portal.

    See: https://docs.dolby.io/communications-apis/reference/start-conference-remix

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        layout_url: Overwrites the layout URL configuration:
            null: uses the layout URL configured in the dashboard
                (if no URL is set in the dashboard, then uses the Dolby.io default)
            default: uses the Dolby.io default layout
            URL string: uses this layout URL
        layout_name: Defines a name for the given layout URL, which makes layout identification
            easier for customers especially when the layout URL is not explicit.

    Returns:
        A :class:`RemixStatus` object that represents the status of the remix.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_url_v2()}/conferences/mix/{conference_id}/remix/start'

    payload = {}
    add_if_not_none(payload, 'layoutUrl', layout_url)
    add_if_not_none(payload, 'layoutName', layout_name)

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=url,
            payload=payload,
        )

    return RemixStatus(json_response)

async def get_status(
        access_token: str,
        conference_id: str,
    ) -> RemixStatus:
    r"""
    Get the status of a current mixing job. You must use this API if the conference is protected
    using enhanced conference access control.

    See: https://docs.dolby.io/communications-apis/reference/get-conference-remix-status

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Returns:
        A :class:`RemixStatus` object that represents the status of the remix.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_url_v2()}/conferences/mix/{conference_id}/remix/status'

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
        )

    return RemixStatus(json_response)



"""
dolbyio_rest_apis.media.enhance
~~~~~~~~~~~~~~~

This module contains the functions to work with the Enhance APIs.
"""

from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.enhance_response import EnhanceJob

async def start(
        access_token: str,
        job_content: str,
    ) -> str or None:
    r"""
    Starts enhancing to improve your media.

    The input location for your source media file as well as the output location for the processed result are required.

    This is an asynchronous operation so you will receive a job_id where you can retrieve the results when enhancement is complete.

    There are additional optional parameters that can be provided to control and select the type of
    enhancements made. See the samples for some examples of what requests and responses look like.

    See: https://docs.dolby.io/media-apis/reference/media-enhance-post

    Args:
        access_token: Access token to use for authentication.
        job_content: Content of the job description as a JSON payload.
            You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-enhance-post

    Returns:
        The job identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/enhance',
            payload=job_content,
        )

    if 'job_id' in json_response:
        return json_response['job_id']

async def get_results(
        access_token: str,
        job_id: str,
    ) -> EnhanceJob:
    r"""
    Gets Enhance Results

    For a given job_id, this method will check if the processing task has completed and return the enhanced results.

    When the status is Success you'll be able to retrieve your result from the output location you provided in the original POST.

    See: https://docs.dolby.io/media-apis/reference/media-enhance-get

    Args:
        access_token: Access token to use for authentication.
        job_id: The job identifier.

    Returns:
        An :class:`EnhanceJob` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/enhance',
            params=params
        )

    return EnhanceJob(job_id, json_response)



"""
dolbyio_rest_apis.core.http_context
~~~~~~~~~~~~~~~

This module contains the HTTP Context class.
"""

import aiofiles
from aiohttp import BasicAuth, ClientResponse, ClientTimeout, ServerTimeoutError, ContentTypeError
from aiohttp_retry import RetryClient, JitterRetry
import certifi
import datetime
import importlib
import logging
import platform
from .rate_limiter import RATE_LIMITER
import ssl
from typing import Any, Mapping, Optional, Type
from types import TracebackType

TOTAL_REQUEST_TIMEOUT: int = 60 # seconds
TOTAL_REQUEST_DOWNLOAD_FILE_TIMEOUT: int = 30 * 60 # 30 minutes
CONNECT_REQUEST_TIMEOUT: int = 25 # seconds

RETRY_MAX_ATTEMPTS: int = 3
RETRY_START_TIMEOUT: float = 1.0

PACKAGE_NAME = 'dolbyio_rest_apis'

class HttpContext:
    """
    HTTP Context used to send HTTP requests.
    """

    def __init__(self):
        self._logger = logging.getLogger(HttpContext.__name__)

        retry_options = JitterRetry(
            attempts=RETRY_MAX_ATTEMPTS,
            start_timeout=RETRY_START_TIMEOUT,
            random_interval_size=1.0,
            statuses=[ 502, 503 ],
            exceptions=[ ServerTimeoutError ],
        )

        self._session = RetryClient(
            raise_for_status=False,
            retry_options=retry_options,
        )

    async def close(self):
        if not self._session is None:
            await self._session.close()
            self._session = None

    def __enter__(self) -> None:
        raise TypeError('Use async with instead')

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        # __exit__ should exist in pair with __enter__ but never executed
        pass  # pragma: no cover

    async def __aenter__(self) -> 'HttpContext':
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        await self.close()

    async def _download_file(
            self,
            url: str,
            headers: Mapping[str, str],
            file_path: str,
            params: Mapping[str, str]=None,
        ):
        self._logger.debug('GET %s', url)

        version = importlib.metadata.version(PACKAGE_NAME)
        user_agent = f'DolbyIoRestApiSdk/{version}; Python/{platform.python_version()}'
        if headers is None:
            headers = {
                'User-Agent': user_agent,
            }
        else:
            headers['User-Agent'] = user_agent

        sslcontext = ssl.create_default_context(cafile=certifi.where())

        async with self._session.get(
            url,
            params=params,
            headers=headers,
            ssl=sslcontext,
            timeout=ClientTimeout(total=TOTAL_REQUEST_DOWNLOAD_FILE_TIMEOUT, connect=CONNECT_REQUEST_TIMEOUT),
        ) as http_response:
            await self._raise_for_status(http_response)

            total_kb = 0

            async with aiofiles.open(file_path, mode='wb') as output_file:
                async for chunk in http_response.content.iter_chunked(1024):
                    total_kb += 1
                    if total_kb % 10240 == 0:
                        # Only print every 10 MB
                        self._logger.debug('Downloading %s - %.1f MB', file_path, total_kb / 1024)
                    await output_file.write(chunk)

            self._logger.debug('Downloaded %s - %.1f MB', file_path, total_kb / 1024)

    async def _upload_file(
            self,
            url: str,
            file_path: str,
        ):
        self._logger.debug('PUT %s', url)

        version = importlib.metadata.version(PACKAGE_NAME)
        headers = {
            'User-Agent': f'DolbyIoRestApiSdk/{version}; Python/{platform.python_version()}',
        }

        with open(file_path, 'rb') as input_file:
            sslcontext = ssl.create_default_context(cafile=certifi.where())
            await self._session.put(
                url,
                headers=headers,
                ssl=sslcontext,
                data=input_file,
            )

    async def _send_request(
            self,
            method: str,
            url: str,
            headers: Mapping[str, str],
            params: Mapping[str, str]=None,
            auth: BasicAuth=None,
            data: Any=None,
        ) -> Any or None:
        if params is None:
            self._logger.debug('%s %s', method, url)
        else:
            self._logger.debug('%s %s %s', method, url, params)
        start = datetime.datetime.now()

        version = importlib.metadata.version(PACKAGE_NAME)
        user_agent = f'DolbyIoRestApiSdk/{version}; Python/{platform.python_version()}'
        if headers is None:
            headers = {
                'User-Agent': user_agent,
            }
        else:
            headers['User-Agent'] = user_agent

        try:
            # Use the rate limited to let request going through
            await RATE_LIMITER.wait_until_allowed()

            sslcontext = ssl.create_default_context(cafile=certifi.where())

            async with self._session.request(
                method=method,
                url=url,
                headers=headers,
                params=params,
                auth=auth,
                data=data,
                ssl=sslcontext,
                timeout=ClientTimeout(total=TOTAL_REQUEST_TIMEOUT, connect=CONNECT_REQUEST_TIMEOUT),
            ) as http_response:
                end = datetime.datetime.now()
                span = end - start
                self._logger.debug('Elapsed %.3f seconds', span.total_seconds())

                await self._raise_for_status(http_response)

                return await http_response.json()
        except ContentTypeError:
            return None # No JSON content
        except ServerTimeoutError:
            self._logger.error('Unable to get data from the url %s because of a timeout.', url)
            self._logger.error('Timeout is set to %i seconds.', TOTAL_REQUEST_TIMEOUT)
            raise

    async def _raise_for_status(self, http_response: ClientResponse):
        raise NotImplementedError()



"""
dolbyio_rest_apis.media.models.diagnose_response
~~~~~~~~~~~~~~~

This module contains the Diagnose Response model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default
from .job_response import JobResponse

class DiagnoseJobResult(dict):
    """The :class:`DiagnoseJobResult` object, which represents the result for a diagnose job."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.version = get_value_or_default(dictionary, 'version', None)

class DiagnoseJob(JobResponse):
    """The :class:`DiagnoseJob` object, which represents the result for a diagnose job."""

    def __init__(self, job_id, dictionary: dict):
        JobResponse.__init__(self, job_id, dictionary)

        if 'result' in dictionary:
            self.result = DiagnoseJobResult(dictionary)



"""
dolbyio_rest_apis.media.models.mastering_response
~~~~~~~~~~~~~~~

This module contains the Mastering Response model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default
from .job_response import JobResponse

class MasteringPreviewJobResult(dict):
    """The :class:`MasteringPreviewJobResult` object, which represents the result for a mastering preview job."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.initial_level = get_value_or_default(dictionary, 'initial_level', None)

class MasteringPreviewJob(JobResponse):
    """The :class:`MasteringPreviewJob` object, which represents the result for a mastering preview job."""

    def __init__(self, job_id, dictionary: dict):
        JobResponse.__init__(self, job_id, dictionary)

        if 'result' in dictionary:
            self.result = MasteringPreviewJobResult(dictionary)

class MasteringJobResult(MasteringPreviewJobResult):
    """The :class:`MasteringJobResult` object, which represents the result for a mastering job."""

    def __init__(self, dictionary: dict):
        MasteringPreviewJobResult.__init__(self, dictionary)

        self.final_level = get_value_or_default(dictionary, 'final_level', None)

class MasteringJob(JobResponse):
    """The :class:`MasteringJob` object, which represents the result for a mastering job."""

    def __init__(self, job_id, dictionary: dict):
        JobResponse.__init__(self, job_id, dictionary)

        if 'result' in dictionary:
            self.result = MasteringJobResult(dictionary)



"""
dolbyio_rest_apis.media.diagnose
~~~~~~~~~~~~~~~

This module contains the functions to work with the Diagnose APIs.
"""

from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.diagnose_response import DiagnoseJob

async def start(
        access_token: str,
        job_content: str,
    ) -> str or None:
    r"""
    Starts Diagnosing.

    The Dolby.io Media Analyze Audio Diagnose API provides a quick diagnosis for discovering audio quality issues with your media.

    See: https://docs.dolby.io/media-apis/reference/media-diagnose-post

    Beta API
    This API is being made available as an early preview.
    If you have feedback on how you'd like to use the API please reach out to share your feedback with our team.
    https://dolby.io/contact

    Args:
        access_token: Access token to use for authentication.
        job_content: Content of the job description as a JSON payload.
            You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-diagnose-post

    Returns:
        The job identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    print('''Beta API
    This API is being made available as an early preview.
    If you have feedback on how you\'d like to use the API please reach out to share your feedback with our team.
    https://dolby.io/contact''')

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/diagnose',
            payload=job_content,
        )

    if 'job_id' in json_response:
        return json_response['job_id']

async def get_results(
        access_token: str,
        job_id: str,
    ) -> DiagnoseJob:
    r"""
    Gets Enhance Results

    For a given job_id, this method will check if the processing task has completed and return the enhanced results.

    When the status is Success you'll be able to retrieve your result from the output location you provided in the original POST.

    See: https://docs.dolby.io/media-apis/reference/media-diagnose-get

    Beta API
    This API is being made available as an early preview.
    If you have feedback on how you'd like to use the API please reach out to share your feedback with our team.
    https://dolby.io/contact

    Args:
        access_token: Access token to use for authentication.
        job_id: The job identifier.

    Returns:
        An :class:`DiagnoseJob` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/diagnose',
            params=params
        )

    return DiagnoseJob(job_id, json_response)



"""
dolbyio_rest_apis.media.io
~~~~~~~~~~~~~~~

This module contains the functions to work with the IO APIs.
"""

from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext

async def get_upload_url(
        access_token: str,
        dlb_url: str,
    ) -> str or None:
    r"""
    Start Media Input

    To use the Dolby provided temporary storage is a two step process.

    You start by declaring a dlb:// url that you can reference in any other Media API calls.
    The response will provide a url where you can put your media.
    This allows you to use the dlb:// url as a short-cut for a temporary storage location.

    You'll be returned a pre-signed url you can use to PUT and upload your media file.
    The temporary storage should allow you to read and write to the dlb:// locations for a period of at least 24 hours before it is removed.

    See: https://docs.dolby.io/media-apis/reference/media-input-post

    Args:
        access_token: Access token to use for authentication.
        dlb_url: The `url` should be in the form `dlb://object-key` where the object-key can be any alpha-numeric string.
            The object-key is unique to your account API Key so there is no risk of collision with other users.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    payload = {
        'url': dlb_url
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/input',
            payload=payload
        )

    if 'url' in json_response:
        return json_response['url']

async def upload_file(
        upload_url: str,
        file_path: str,
    ) -> None:
    r"""
    Upload a file.

    Args:
        upload_url: URL where to upload the file to.
        file_path: Local file path to upload.

    Raises:
        HTTPError: If one occurred.
    """
    async with MediaHttpContext() as http_context:
        await http_context.upload(
            upload_url=upload_url,
            file_path=file_path,
        )

async def download_file(
        access_token: str,
        dlb_url: str,
        file_path: str,
    ) -> None:
    r"""
    Start Media Download

    You can download media you previously uploaded with /media/input or media that was generated through another Dolby Media API.

    The temporary storage should allow you to read and write to the dlb:// locations for a period of at least 24 hours before it is removed.

    See: https://docs.dolby.io/media-apis/reference/media-output-get

    Args:
        access_token: Access token to use for authentication.
        dlb_url: The `url` should be in the form `dlb://object-key` where the object-key can be any alpha-numeric string.
            The object-key is unique to your account API Key so there is no risk of collision with other users.
        file_path: Local file path where to download the file to.

    Raises:
        HTTPError: If one occurred.
    """
    params = {
        'url': dlb_url,
    }

    async with MediaHttpContext() as http_context:
        await http_context.download(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/output',
            file_path=file_path,
            params=params,
        )



"""
dolbyio_rest_apis.media.webhooks
~~~~~~~~~~~~~~~

This module contains the functions to work with the Platform Webhooks APIs.
"""

from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.webhook import Webhook
from typing import Any, Dict

async def register_webhook(
        access_token: str,
        url: str,
        headers: Dict[str, Any]=None,
    ) -> str or None:
    r"""
    Registers a webhook that is triggered when a job completes.

    See: https://docs.dolby.io/media-apis/reference/media-webhook-post

    Args:
        access_token: Access token to use for authentication.
        url: The callback url that will be called when job execution completes.
        headers: (Optional) Headers to include in the webhook call.

    Returns:
        The webhook identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    payload = {
        'callback': {
            'url': url,
        }
    }
    add_if_not_none(payload['callback'], 'headers', headers)

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/webhooks',
            payload=payload,
        )

    if 'webhook_id' in json_response:
        return json_response['webhook_id']

async def update_webhook(
        access_token: str,
        webhook_id: str,
        url: str,
        headers: Dict[str, Any]=None,
    ):
    r"""
    Updates the previously registered webhook configuration.

    See: https://docs.dolby.io/media-apis/reference/media-webhook-put

    Args:
        access_token: Access token to use for authentication.
        webhook_id: Use the `webhook_id` returned from a previous GET, POST
            or PUT response to retrieve the webhook configuration.
        url: The callback url that will be called when job execution completes.
        headers: (Optional) Headers to include in the webhook call.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'id': webhook_id
    }

    payload = {
        'callback': {
            'url': url,
        }
    }
    add_if_not_none(payload['callback'], 'headers', headers)

    async with MediaHttpContext() as http_context:
        await http_context.requests_put(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/webhooks',
            params=params,
            payload=payload,
        )

async def retrieve_webhook(
        access_token: str,
        webhook_id: str,
    ) -> Webhook:
    r"""
    Retrieves the previously registered webhook configuration.

    See: https://docs.dolby.io/media-apis/reference/media-webhook-get

    Args:
        access_token: Access token to use for authentication.
        url: The callback url that will be called when job execution completes.
        headers: (Optional) Headers to include in the webhook call.

    Returns:
        A :class:`Webhook` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'id': webhook_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/webhooks',
            params=params,
        )

    return Webhook(json_response)

async def delete_webhook(
        access_token: str,
        webhook_id: str,
    ) -> str or None:
    r"""
    Deletes a previously registered webhook configuration.

    See: https://docs.dolby.io/media-apis/reference/media-webhook-delete

    Args:
        access_token: Access token to use for authentication.
        webhook_id: Use the `webhook_id` returned from a previous GET, POST
            or PUT response to retrieve the webhook configuration.

    Returns:
        The webhook identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'id': webhook_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_delete(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/webhooks',
            params=params,
        )

    if 'webhook_id' in json_response:
        return json_response['webhook_id']



"""
dolbyio_rest_apis.communications.monitor.models
~~~~~~~~~~~~~~~

This module contains the models used by the Dolby.io APIs.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default, in_and_not_none
from typing import List

class PagedResponse(dict):
    """Representation of a paged response."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.first = get_value_or_default(self, 'first', None)
        self.next = get_value_or_default(self, 'next', None)

class ConferenceOwner(dict):
    """Representation of a Conference Owner."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.user_id = get_value_or_default(self, 'userID', None)

        if in_and_not_none(self, 'metadata'):
            self.metadata = UserMetadata(self['metadata'])

class ConferenceStatisticsMaxParticipants(dict):
    """Representation of a Conference Statistics Max Participants."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.user = get_value_or_default(self, 'USER', 0)
        self.listener = get_value_or_default(self, 'LISTENER', 0)
        self.mixer = get_value_or_default(self, 'MIXER', 0)
        self.pstn = get_value_or_default(self, 'PSTN', 0)

class ConferenceParticipant(dict):
    """Representation of a Conference Participant."""

    def __init__(self, user_id, dictionary: dict):
        self.user_id = user_id
        dict.__init__(self, dictionary)

        #if in_and_not_none(self, 'connections'):
        #    self.connections = ConferenceOwner(self['connections'])

        #if in_and_not_none(self, 'stats'):
        #    self.stats = ConferenceStatistics(self['stats'])

class ConferenceParticipants(PagedResponse):
    """Representation of a Conference participants."""

    def __init__(self, dictionary: dict):
        PagedResponse.__init__(self, dictionary)

        self.participants: List[ConferenceParticipant] = []
        if in_and_not_none(self, 'participants'):
            for key in self['participants'].keys():
                participant = ConferenceParticipant(key, self['participants'][key])
                self.participants.append(participant)

class ConferenceStatisticsMaxRate(dict):
    """Representation of a Conference Statistics Max Rate."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.dtls = get_value_or_default(self, 'DTLS', 0)
        self.rtcp = get_value_or_default(self, 'RTCP', 0)
        self.rtp = get_value_or_default(self, 'RTP', 0)
        self.stun = get_value_or_default(self, 'STUN', 0)

class ConferenceStatisticsMaxStreams(dict):
    """Representation of a Conference Statistics Max Streams."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.audio = get_value_or_default(self, 'AUDIO', 0)
        self.video = get_value_or_default(self, 'VIDEO', 0)
        self.screenshare = get_value_or_default(self, 'SCREENSHARE', 0)

class ConferenceStatisticsNetwork(dict):
    """Representation of a Conference Statistics Network."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        if in_and_not_none(self, 'maxRxBytesRate'):
            self.max_rx_bytes_rate = ConferenceStatisticsMaxRate(self['maxRxBytesRate'])
        if in_and_not_none(self, 'maxRxPacketsRate'):
            self.max_rx_packets_rate = ConferenceStatisticsMaxRate(self['maxRxPacketsRate'])
        if in_and_not_none(self, 'maxRxStreams'):
            self.max_rx_streams = ConferenceStatisticsMaxStreams(self['maxRxStreams'])
        if in_and_not_none(self, 'maxTxBytesRate'):
            self.max_tx_bytes_rate = ConferenceStatisticsMaxRate(self['maxTxBytesRate'])
        if in_and_not_none(self, 'maxTxPacketsRate'):
            self.max_tx_packets_rate = ConferenceStatisticsMaxRate(self['maxTxPacketsRate'])
        if in_and_not_none(self, 'maxTxStreams'):
            self.max_tx_streams = ConferenceStatisticsMaxStreams(self['maxTxStreams'])

class ConferenceStatistics(dict):
    """Representation of a Conference Statistics."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        if in_and_not_none(self, 'maxParticipants'):
            self.max_participants = ConferenceStatisticsMaxParticipants(self['maxParticipants'])

        if in_and_not_none(self, 'network'):
            self.network = ConferenceStatisticsNetwork(self['network'])

class ConferenceSummary(dict):
    """Representation of a Conference Summary."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.conf_id = get_value_or_default(self, 'confId', None)
        self.alias = get_value_or_default(self, 'alias', None)
        self.region = get_value_or_default(self, 'region', None)
        self.start = get_value_or_default(self, 'start', 0)
        self.live = get_value_or_default(self, 'live', False)
        self.end = get_value_or_default(self, 'end', 0)
        self.duration = get_value_or_default(self, 'duration', 0)
        self.type = get_value_or_default(self, 'type', None)
        self.presence_duration = get_value_or_default(self, 'presenceDuration', 0)
        self.recording_duration = get_value_or_default(self, 'recordingDuration', 0)
        self.mixer_live_recording = get_value_or_default(self, 'mixerLiveRecording', 0)
        self.mixer_hls_streaming = get_value_or_default(self, 'mixerHlsStreaming', 0)
        self.mixer_rtmp_streaming = get_value_or_default(self, 'mixerRtmpStreaming', 0)
        self.nb_users = get_value_or_default(self, 'nbUsers', 0)
        self.nb_listeners = get_value_or_default(self, 'nbListeners', 0)
        self.nb_pstn = get_value_or_default(self, 'nbPstn', 0)

        if in_and_not_none(self, 'owner'):
            self.mix = ConferenceOwner(self['owner'])

        if in_and_not_none(self, 'statistics'):
            self.statistics = ConferenceStatistics(self['statistics'])

class GetConferencesResponse(PagedResponse):
    """Representation of a Conferences response."""

    def __init__(self, dictionary: dict):
        PagedResponse.__init__(self, dictionary)

        self.conferences: List[ConferenceSummary] = []
        if in_and_not_none(self, 'conferences'):
            for conference in self['conferences']:
                self.conferences.append(ConferenceSummary(conference))

class RecordingMix(dict):
    """Representation of a Recording Mix."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.mp4 = get_value_or_default(self, 'mp4', 0)
        self.mp3 = get_value_or_default(self, 'mp3', 0)
        self.region = get_value_or_default(self, 'region', None)

class UserMetadata(dict):
    """Representation of a User Metadata."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.user_id = get_value_or_default(self, 'userID', None)
        self.external_name = get_value_or_default(self, 'externalName', None)
        self.external_id = get_value_or_default(self, 'externalId', None)
        self.external_photo_url = get_value_or_default(self, 'externalPhotoUrl', None)
        self.ip_address = get_value_or_default(self, 'ipAddress', None)

class RecordingSplit(dict):
    """Representation of a Recording Split."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.start_time = get_value_or_default(self, 'startTime', 0)
        self.duration = get_value_or_default(self, 'duration', 0)
        self.size = get_value_or_default(self, 'size', 0)
        self.file_name = get_value_or_default(self, 'fileName', None)
        self.url = get_value_or_default(self, 'url', None)

        if in_and_not_none(self, 'metadata'):
            self.metadata = UserMetadata(self['metadata'])

class RecordingRecord(dict):
    """Representation of a Recording Record."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.start_time = get_value_or_default(self, 'startTime', 0)
        self.duration = get_value_or_default(self, 'duration', 0)
        self.size = get_value_or_default(self, 'size', 0)
        self.file_name = get_value_or_default(self, 'fileName', None)
        self.url = get_value_or_default(self, 'url', None)

        self.splits = []
        if in_and_not_none(self, 'splits'):
            for split in self['splits']:
                self.splits.append(RecordingSplit(split))

class RecordingAudio(dict):
    """Representation of a Recording Audio."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.region = get_value_or_default(self, 'region', None)

        if in_and_not_none(self, 'mix'):
            self.mix = RecordingMix(self['mix'])

        self.records = []
        if in_and_not_none(self, 'records'):
            for record in self['records']:
                self.records.append(RecordingRecord(record))

class Recording(dict):
    """Representation of a Recording."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.conf_id = get_value_or_default(self, 'confId', None)
        self.alias = get_value_or_default(self, 'alias', None)
        self.duration = get_value_or_default(self, 'duration', 0)
        self.ts = get_value_or_default(self, 'ts', 0)
        self.region = get_value_or_default(self, 'region', None)

        if in_and_not_none(self, 'mix'):
            self.mix = RecordingMix(self['mix'])

        if in_and_not_none(self, 'audio'):
            self.audio = RecordingAudio(self['audio'])

class GetRecordingsResponse(PagedResponse):
    """Representation of a Recordings response."""

    def __init__(self, dictionary: dict):
        PagedResponse.__init__(self, dictionary)

        self.recordings = []
        if in_and_not_none(self, 'recordings'):
            for recording in self['recordings']:
                self.recordings.append(Recording(recording))

class DolbyVoiceRecording(dict):
    """Representation of a Dolby Voice Recording."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.region = get_value_or_default(self, 'region', None)
        self.conf_id = None
        self.conf_alias = None
        if in_and_not_none(self, 'conference'):
            self.conf_id = get_value_or_default(self, 'confId', None)
            self.conf_alias = get_value_or_default(self, 'confAlias', None)

        self.records = []
        if in_and_not_none(self, 'records'):
            for record in self['records']:
                self.records.append(RecordingRecord(record))

class WebHookResponse(dict):
    """Representation of a WebHook event response."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.status = get_value_or_default(self, 'status', None)
        self.headers = get_value_or_default(self, 'headers', None)

class WebHook(dict):
    """Representation of a WebHook event."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.id = get_value_or_default(self, 'id', None)
        self.webhook = get_value_or_default(self, 'webhook', None)
        self.url = get_value_or_default(self, 'url', None)
        self.conf_id = get_value_or_default(self, 'confId', None)
        self.third_party_id = get_value_or_default(self, 'thirdPartyId', None)
        self.ts = get_value_or_default(self, 'ts', None)

        if in_and_not_none(self, 'response'):
            self.response = WebHookResponse(self['response'])

class GetWebHookResponse(PagedResponse):
    """Representation of a WebHook response."""

    def __init__(self, dictionary: dict):
        PagedResponse.__init__(self, dictionary)

        self.webhooks = []
        if in_and_not_none(self, 'webhooks'):
            for wbk in self['webhooks']:
                self.webhooks.append(WebHook(wbk))



"""
dolbyio_rest_apis.streaming.subscribe_token
~~~~~~~~~~~~~~~

This module contains the functions to work with the Subscribe Token APIs.
"""

from typing import List
from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.core.urls import get_rts_url
from dolbyio_rest_apis.streaming.internal.http_context import StreamingHttpContext
from dolbyio_rest_apis.streaming.models.subscribe_token import SubscribeToken, UpdateSubscribeToken, CreateSubscribeToken

async def read(
        api_secret: str,
        token_id: int,
    ) -> SubscribeToken:
    async with StreamingHttpContext() as http_context:
        json_response = await http_context.requests_get(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/subscribe_token/{token_id}',
        )

    return SubscribeToken(json_response)

async def delete(
        api_secret: str,
        token_id: int,
    ) -> None:
    async with StreamingHttpContext() as http_context:
        await http_context.requests_delete(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/subscribe_token/{token_id}',
        )

async def update(
        api_secret: str,
        token_id: int,
        token: UpdateSubscribeToken,
    ) -> SubscribeToken:
    payload = {}
    add_if_not_none(payload, 'label', token.label)
    add_if_not_none(payload, 'refreshToken', token.refresh_token)
    add_if_not_none(payload, 'isActive', token.is_active)
    if token.add_token_streams is not None:
        payload['addTokenStreams'] = []
        for stream in token.add_token_streams:
            stream_obj = {
                'streamName': stream.stream_name,
            }
            add_if_not_none(stream_obj, 'isRegex', stream.is_regex)
            payload['addTokenStreams'].append(stream_obj)
    if token.remove_token_streams is not None:
        payload['removeTokenStreams'] = []
        for stream in token.remove_token_streams:
            stream_obj = {
                'streamName': stream.stream_name,
            }
            add_if_not_none(stream_obj, 'isRegex', stream.is_regex)
            payload['removeTokenStreams'].append(stream_obj)
    add_if_not_none(payload, 'updateAllowedOrigins', token.update_allowed_origins)
    add_if_not_none(payload, 'updateAllowedIpAddresses', token.update_allowed_ip_addresses)
    add_if_not_none(payload, 'updateBindIpsOnUsage', token.update_bind_ips_on_usage)
    add_if_not_none(payload, 'updateAllowedCountries', token.update_allowed_countries)
    add_if_not_none(payload, 'updateDeniedCountries', token.update_denied_countries)
    add_if_not_none(payload, 'updateOriginCluster', token.update_origin_cluster)
    async with StreamingHttpContext() as http_context:
        json_response = await http_context.requests_put(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/subscribe_token/{token_id}',
            payload=payload,
        )

    return SubscribeToken(json_response)

async def list_tokens(
        api_secret: str,
        sort_by: str,
        page: int,
        items_on_page: int,
        is_descending: bool,
    ) -> List[SubscribeToken]:
    params = {
        'sortBy': sort_by,
        'page': str(page),
        'itemsOnPage': str(items_on_page),
        'isDescending': str(is_descending),
    }

    async with StreamingHttpContext() as http_context:
        json_response = await http_context.requests_get(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/subscribe_token/list',
            params=params,
        )

    publish_tokens = []
    for token in json_response:
        publish_tokens.append(SubscribeToken(token))
    return publish_tokens

async def create(
        api_secret: str,
        token: CreateSubscribeToken,
    ) -> SubscribeToken:
    payload = {
        'label': token.label,
        'streams': [],
    }
    add_if_not_none(payload, 'expiresOn', token.expires_on)
    for stream in token.streams:
        stream_obj = {
            'streamName': stream.stream_name,
        }
        add_if_not_none(stream_obj, 'isRegex', stream.is_regex)
        payload['streams'].append(stream_obj)
    add_if_not_none(payload, 'allowedOrigins', token.allowed_origins)
    add_if_not_none(payload, 'allowedIpAddresses', token.allowed_ip_addresses)
    add_if_not_none(payload, 'bindIpsOnUsage', token.bind_ips_on_usage)
    add_if_not_none(payload, 'allowedCountries', token.allowed_countries)
    add_if_not_none(payload, 'deniedCountries', token.denied_countries)
    add_if_not_none(payload, 'originCluster', token.origin_cluster)

    async with StreamingHttpContext() as http_context:
        json_response = await http_context.requests_post(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/subscribe_token',
            payload=token,
        )

    return SubscribeToken(json_response)



"""
dolbyio_rest_apis.communications.monitor.webhooks
~~~~~~~~~~~~~~~

This module contains the functions to work with the monitor API related to webhooks.
"""

from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.communications.monitor.models import GetWebHookResponse, WebHook
from dolbyio_rest_apis.core.urls import get_comms_monitor_url
from typing import Any, List

async def get_events(
        access_token: str,
        conference_id: str=None,
        tr_from: int=0,
        tr_to: int=9999999999999,
        maximum: int=100,
        start: str=None,
        filter_type: str=None,
    ) -> GetWebHookResponse:
    r"""
    Get a list of Webhook events sent, during a specific time range.
    The list includes associated endpoint response codes and headers.

    See: https://docs.dolby.io/communications-apis/reference/get-webhooks

    Args:
        access_token: Access token to use for authentication.
        conference_id: (Optional) Identifier of the conference.
        tr_from: (Optional) The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: (Optional) The end of the time range (in milliseconds that have elapsed since epoch).
        maximum: (Optional) The maximum number of displayed results.
            We recommend setting the proper value of this parameter to shorten the response time.
        start: (Optional) When the results span multiple pages, use this option to navigate through pages.
            By default, only the max number of results is displayed. To see the next results,
            set the start parameter to the value of the next key returned in the previous response.
        filter_type: (Optional) The Webhook event type or an expression of its type (for example `Recording.Live.InProgress` or `Rec.*`).
            The default value of the type parameter returns all types of Webhooks.

    Returns:
        A :class:`GetWebHookResponse` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/'
    if not conference_id is None:
        url += f'conferences/{conference_id}/'
    url += 'webhooks'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': maximum,
    }

    if not filter_type is None:
        params['type'] = filter_type
    if not start is None:
        params['start'] = start

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
            params=params,
        )

    return GetWebHookResponse(json_response)

async def get_all_events(
        access_token: str,
        conference_id: str=None,
        tr_from: int=0,
        tr_to: int=9999999999999,
        page_size: int=100,
        filter_type: str=None,
    ) -> List[WebHook]:
    r"""
    Get a list of all Webhook events sent, during a specific time range.
    The list includes associated endpoint response codes and headers.

    See: https://docs.dolby.io/communications-apis/reference/get-webhooks

    Args:
        access_token: Access token to use for authentication.
        conference_id: (Optional) Identifier of the conference.
        tr_from: (Optional) The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: (Optional) The end of the time range (in milliseconds that have elapsed since epoch).
        page_size: (Optional) Number of elements to return per page.
        filter_type: (Optional) The Webhook event type or an expression of its type (for example `Recording.Live.InProgress` or `Rec.*`).
            The default value of the type parameter returns all types of Webhooks.

    Returns:
        A list of :class:`WebHook` objects.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/'
    if not conference_id is None:
        url += f'conferences/{conference_id}/'
    url += 'webhooks'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': page_size,
    }

    if not filter_type is None:
        params['type'] = filter_type

    async with CommunicationsHttpContext() as http_context:
        elements: List[Any] = await http_context.requests_get_all(
            access_token=access_token,
            url=url,
            params=params,
            property_name='webhooks',
            page_size=page_size
        )

    webhooks: List[WebHook] = []
    for element in elements:
        webhook = WebHook(element)
        webhooks.append(webhook)

    return webhooks



"""
dolbyio_rest_apis.media.transcode
~~~~~~~~~~~~~~~

This module contains the functions to work with the Transcode APIs.
"""

from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.mastering_response import MasteringPreviewJob, MasteringJob

async def start_preview(
        access_token: str,
        job_content: str,
    ) -> str or None:
    r"""
    Starts mastering preview to improve your music.

    The inputs location for your source media file as well as the outputs locations for the processed media files are required.

    A preset applies dynamic EQ processing to shape your music to match a desired sound.
    There are also additional optional parameters that can be provided to control the mastering output.

    A segment object specifying preview start may optionally be provided.

    This is an asynchronous operation. You receive a job_id that you use to retrieve the results when the mastering is complete.

    To learn more, see the example requests and responses.

    See: https://docs.dolby.io/media-apis/reference/media-music-mastering-preview-post

    Args:
        access_token: Access token to use for authentication.
        job_content: Content of the job description as a JSON payload.
            You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-music-mastering-preview-post

    Returns:
        The job identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/master/preview',
            payload=job_content,
        )

    if 'job_id' in json_response:
        return json_response['job_id']

async def get_preview_results(
        access_token: str,
        job_id: str,
    ) -> MasteringPreviewJob:
    r"""
    Gets Mastering Preview Results

    For a given job_id, this method will check if the mastering task has completed.
    When the status is Success you'll be able to retrieve your results from the outputs locations you provided in the original POST.

    See: https://docs.dolby.io/media-apis/reference/media-music-mastering-preview-get

    Args:
        access_token: Access token to use for authentication.
        job_id: The job identifier.

    Returns:
        An :class:`MasteringPreviewJob` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/master/preview',
            params=params
        )

    return MasteringPreviewJob(job_id, json_response)

async def start(
        access_token: str,
        job_content: str,
    ) -> str or None:
    r"""
    Starts mastering to improve your music.

    The inputs location for your source media file as well as the outputs location for the processed media file are required.

    A preset applies dynamic EQ processing to shape your music to match a desired sound.
    There are also additional optional parameters that can be provided to control the mastering output.

    This is an asynchronous operation. You receive a job_id that you use to retrieve the results when the mastering is complete.

    To learn more, see the example requests and responses.

    See: https://docs.dolby.io/media-apis/reference/media-music-mastering-post

    Args:
        access_token: Access token to use for authentication.
        job_content: Content of the job description as a JSON payload.
            You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-music-mastering-post

    Returns:
        The job identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/master',
            payload=job_content,
        )

    if 'job_id' in json_response:
        return json_response['job_id']

async def get_results(
        access_token: str,
        job_id: str,
    ) -> MasteringJob:
    r"""
    Gets Mastering Results

    For a given job_id, this method will check if the mastering task has completed.
    When the status is Success you'll be able to retrieve your results from the outputs locations you provided in the original POST.

    See: https://docs.dolby.io/media-apis/reference/media-music-mastering-get

    Args:
        access_token: Access token to use for authentication.
        job_id: The job identifier.

    Returns:
        An :class:`MasteringJob` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/master',
            params=params
        )

    return MasteringJob(job_id, json_response)



"""
Setup project
"""

import os
import pathlib
import setuptools

current_path = pathlib.Path(__file__).parent.resolve()

with open(os.path.join(current_path, 'README.md'), 'r', encoding='UTF-8') as f:
    long_description = f.read()

with open(os.path.join(current_path, 'requirements.txt'), 'r', encoding='UTF-8') as f:
    requirements = f.readlines()

setuptools.setup(
    name='dolbyio-rest-apis',
    author='Dolby.io',
    author_email='fabien.lavocat@dolby.com',
    description='A Python wrapper for the Dolby.io REST APIs.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    url='https://github.com/dolbyio/dolbyio-rest-apis-client-python',
    project_urls={
        'Documentation': 'https://docs.dolby.io/communications-apis/reference',
        'Source': 'https://github.com/dolbyio/dolbyio-rest-apis-client-python',
        'Bug Tracker': 'https://github.com/dolbyio/dolbyio-rest-apis-client-python/issues',
    },
    package_dir={'': os.path.join(current_path, 'src')},
    packages=setuptools.find_packages(where=os.path.join(current_path, 'src')),
    python_requires='>=3.7',
    use_scm_version= {
        'local_scheme': 'no-local-version',
        'version_scheme': 'release-branch-semver',
    },
    setup_requires=[
        'setuptools_scm',
    ],
    install_requires=requirements,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3 :: Only',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
    ],
)



"""
dolbyio_rest_apis.communications.monitor.conferences
~~~~~~~~~~~~~~~

This module contains the functions to work with the monitor API related to conferences.
"""

from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.communications.monitor.models import GetConferencesResponse, ConferenceSummary, ConferenceStatistics, ConferenceParticipants, ConferenceParticipant
from dolbyio_rest_apis.core.urls import get_comms_monitor_url
from typing import Any, Dict, List

async def get_conferences(
        access_token: str,
        tr_from: int=0,
        tr_to: int=9999999999999,
        maximum: int=100,
        start: str=None,
        filter_alias: str=None,
        active: bool=False,
        external_id: str=None,
        live_stats: bool=False,
    ) -> GetConferencesResponse:
    r"""
    Get a list of conferences.

    Get a list of conferences that were started in a specific time range, including ongoing conferences.

    Note: Only terminated conferences include a complete summary.
    The summary of ongoing conferences includes the following fields in the response:
    `confId`, `alias`, `region`, `dolbyVoice`, `start`, `live`, `owner`.

    See: https://docs.dolby.io/communications-apis/reference/get-conferences

    Args:
        access_token: Access token to use for authentication.
        tr_from: (Optional) The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: (Optional) The end of the time range (in milliseconds that have elapsed since epoch).
        maximum: (Optional) The maximum number of displayed results.
            We recommend setting the proper value of this parameter to shorten the response time.
        start: (Optional) When the results span multiple pages, use this option to navigate through pages.
            By default, only the max number of results is displayed. To see the next results,
            set the start parameter to the value of the next key returned in the previous response.
        filter_alias: (Optional) Search conferences using Alias. Use regular expression to search for conferences with similar aliases.
            For example:
            - Use `foobar` to get all conferences with alias foobar.
            - Use `.*foobar` to get all conferences with alias ending with foobar.
            - Use `foobar.*` to get all conferences with alias starting with foobar
            - Use `.*foobar.*` to get all conferences with alias containing foobar.
            - Use `.*2019.*|.*2020.*` to get all conferences with alias containing either 2019 or 2020.
        active: (Optional) Search for ongoing references (`true`) or all conferences (`false`).
        external_id: (Optional) The external ID of the participant who created the conference.
        live_stats: (Optional) For live conferences, the number of `user`, `listener`, and `pstn` participants.

    Returns:
        A :class:`GetConferencesResponse` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/conferences'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': maximum,
        'active': str(active),
        'livestats': str(live_stats),
    }

    if not start is None:
        params['start'] = start
    if not filter_alias is None:
        params['alias'] = filter_alias
    if not external_id is None:
        params['exid'] = external_id

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
            params=params,
        )

    return GetConferencesResponse(json_response)

async def get_all_conferences(
        access_token: str,
        tr_from: int=0,
        tr_to: int=9999999999999,
        page_size: int=100,
        filter_alias: str=None,
        active: bool=False,
        external_id: str=None,
        live_stats: bool=False,
    ) -> List[ConferenceSummary]:
    r"""
    Get a list of all conferences.

    Get a list of all conferences that were started in a specific time range, including ongoing conferences.

    Note: Only terminated conferences include a complete summary.
    The summary of ongoing conferences includes the following fields in the response:
    `confId`, `alias`, `region`, `dolbyVoice`, `start`, `live`, `owner`.

    See: https://docs.dolby.io/communications-apis/reference/get-conferences

    Args:
        access_token: Access token to use for authentication.
        tr_from: (Optional) The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: (Optional) The end of the time range (in milliseconds that have elapsed since epoch).
        page_size: (Optional) Number of elements to return per page.
        filter_alias: (Optional) Search conferences using Alias. Use regular expression to search for conferences with similar aliases.
            For example:
            - Use `foobar` to get all conferences with alias foobar.
            - Use `.*foobar` to get all conferences with alias ending with foobar.
            - Use `foobar.*` to get all conferences with alias starting with foobar
            - Use `.*foobar.*` to get all conferences with alias containing foobar.
            - Use `.*2019.*|.*2020.*` to get all conferences with alias containing either 2019 or 2020.
        active: (Optional) Search for ongoing references (`true`) or all conferences (`false`).
        external_id: (Optional) The external ID of the participant who created the conference.
        live_stats: (Optional) For live conferences, the number of `user`, `listener`, and `pstn` participants.

    Returns:
        A list of :class:`Conference` objects.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/conferences'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': page_size,
        'active': str(active),
        'livestats': str(live_stats),
    }

    if not filter_alias is None:
        params['alias'] = filter_alias
    if not external_id is None:
        params['exid'] = external_id

    async with CommunicationsHttpContext() as http_context:
        elements: List[Any] = await http_context.requests_get_all(
            access_token=access_token,
            url=url,
            params=params,
            property_name='conferences',
            page_size=page_size
        )

    conferences: List[ConferenceSummary] = []
    for element in elements:
        conference = ConferenceSummary(element)
        conferences.append(conference)

    return conferences

async def get_conference(
        access_token: str,
        conference_id: str,
        live_stats: bool=False,
    ) -> ConferenceSummary:
    r"""
    Get a summary of a conference.

    Note: Only terminated conferences include a complete summary.
    The summary of ongoing conferences includes the following fields in the response:
    `confId`, `alias`, `region`, `dolbyVoice`, `start`, `live`, `owner`.

    See: https://docs.dolby.io/communications-apis/reference/get-conference-summary

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        live_stats: (Optional) For live conferences, the number of `user`, `listener`, and `pstn` participants.

    Returns:
        A :class:`ConferenceSummary` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    url = f'{get_comms_monitor_url()}/conferences/{conference_id}'

    params = {
        'livestats': str(live_stats),
    }

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
            params=params,
        )

    return ConferenceSummary(json_response)

async def get_conference_statistics(
        access_token: str,
        conference_id: str,
    ) -> ConferenceStatistics:
    r"""
    Get a conference statistics.

    Get statistics of a terminated conference.
    The statistics include the maximum number of participants present during a conference
    and the maximum number of the transmitted and received packets, bytes, and streams.

    Note: The statistics are available only for terminated conferences.

    See: https://docs.dolby.io/communications-apis/reference/get-conference-statistics

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        live_stats: For live conferences, the number of `user`, `listener`, and `pstn` participants.

    Returns:
        A :class:`ConferenceStatistics` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    url = f'{get_comms_monitor_url()}/conferences/{conference_id}/statistics'

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
        )

    return ConferenceStatistics(json_response)

async def get_conference_participants(
        access_token: str,
        conference_id: str,
        participant_type: str=None,
        tr_from: int=0,
        tr_to: int=9999999999999,
        maximum: int=100,
        start: str=None,
    ) -> ConferenceParticipants:
    r"""
    Get information about conference participants.

    Get statistics and connection details of all participants in a conference.
    Optionally limit the search result with a specific time range.

    See: https://docs.dolby.io/communications-apis/reference/get-info-conference-participants

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        participant_type: (Optional) The conference participant type.
            - `user` - a participant who can send and receive video/audio stream to/from the conference.
            - `listener` - a participant who can only receive video/audio stream from the conference.
            - `pstn` - a participant who connected to the conference using PSTN (telephony network).
            - `mixer` - an internal type indicating a mixer connection to the conference.
        tr_from: (Optional) The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: (Optional) The end of the time range (in milliseconds that have elapsed since epoch).
        maximum: (Optional) The maximum number of displayed results.
            We recommend setting the proper value of this parameter to shorten the response time.
        start: (Optional) When the results span multiple pages, use this option to navigate through pages.
            By default, only the max number of results is displayed. To see the next results,
            set the start parameter to the value of the next key returned in the previous response.

    Returns:
        A :class:`ConferenceParticipants` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/conferences/{conference_id}/participants'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': maximum,
    }
    if not participant_type is None:
        params['type'] = participant_type
    if not start is None:
        params['start'] = start

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
            params=params,
        )

        return ConferenceParticipants(json_response)

async def get_all_conference_participants(
        access_token: str,
        conference_id: str,
        participant_type: str=None,
        tr_from: int=0,
        tr_to: int=9999999999999,
        page_size: int=100,
    ) -> Dict[str, ConferenceParticipant]:
    r"""
    Get information about all conference participants.

    Get statistics and connection details of all participants in a conference.
    Optionally limit the search result with a specific time range.

    See: https://docs.dolby.io/communications-apis/reference/get-info-conference-participants

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        participant_type: (Optional) The conference participant type.
            - `user` - a participant who can send and receive video/audio stream to/from the conference.
            - `listener` - a participant who can only receive video/audio stream from the conference.
            - `pstn` - a participant who connected to the conference using PSTN (telephony network).
            - `mixer` - an internal type indicating a mixer connection to the conference.
        tr_from: (Optional) The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: (Optional) The end of the time range (in milliseconds that have elapsed since epoch).
        page_size: (Optional) Number of elements to return per page.

    Returns:
        A list of :class:`ConferenceParticipant` objects.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/conferences/{conference_id}/participants'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': page_size,
    }
    if not participant_type is None:
        params['type'] = participant_type

    participants: Dict[str, ConferenceParticipant] = {}

    async with CommunicationsHttpContext() as http_context:
        while True:
            json_response = await http_context.requests_get(
                access_token=access_token,
                url=url,
                params=params,
            )

            if 'participants' in json_response:
                sub_result = json_response['participants']
                for participant_id in sub_result:
                    participant = ConferenceParticipant(participant_id, sub_result[participant_id])
                    participants[participant.user_id] = participant

                if len(sub_result) < page_size:
                    break

            if not 'next' in json_response:
                break

            params['start'] = json_response['next']
            if params['start'] is None or params['start'] == '':
                break

    return participants

async def get_conference_participant(
        access_token: str,
        conference_id: str,
        participant_id: str,
        participant_type: str=None,
        tr_from: int=0,
        tr_to: int=9999999999999,
        maximum: int=100,
        start: str=None,
    ) -> ConferenceParticipant:
    r"""
    Get information about a specific conference participant.

    Gets the statistics and connection details of a conference participant, during a specific time range.

    See: https://docs.dolby.io/communications-apis/reference/get-info-conference-participant

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        participant_id: Identifier of the participant.
        participant_type: (Optional) The conference participant type.
            - `user` - a participant who can send and receive video/audio stream to/from the conference.
            - `listener` - a participant who can only receive video/audio stream from the conference.
            - `pstn` - a participant who connected to the conference using PSTN (telephony network).
            - `mixer` - an internal type indicating a mixer connection to the conference.
        tr_from: (Optional) The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: (Optional) The end of the time range (in milliseconds that have elapsed since epoch).
        maximum: (Optional) The maximum number of displayed results.
            We recommend setting the proper value of this parameter to shorten the response time.
        start: (Optional) When the results span multiple pages, use this option to navigate through pages.
            By default, only the max number of results is displayed. To see the next results,
            set the start parameter to the value of the next key returned in the previous response.

    Returns:
        A :class:`ConferenceParticipant` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/conferences/{conference_id}/participants/{participant_id}'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': maximum,
    }
    if not participant_type is None:
        params['type'] = participant_type
    if not start is None:
        params['start'] = start

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
            params=params,
        )

        participants = ConferenceParticipants(json_response)
        return participants[participant_id]



"""
dolbyio_rest_apis.media.transcode
~~~~~~~~~~~~~~~

This module contains the functions to work with the Transcode APIs.
"""

from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.transcode_response import TranscodeJob

async def start(
        access_token: str,
        job_content: str,
    ) -> str or None:
    r"""
    Start transcoding to modify the resolutions, bitrates, and formats for your media.

    The inputs location and outputs are required to identify the source material and target for results.

    This is an asynchronous operation so you will receive a job_id to use in calling GET /media/transcode to retrieve the job status.

    There are additional optional parameters that can be provided to further specify the desired output content types.

    See the samples for examples of transcode requests.

    See: https://docs.dolby.io/media-apis/reference/media-transcode-post

    Args:
        access_token: Access token to use for authentication.
        job_content: Content of the job description as a JSON payload.
            You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-transcode-post

    Returns:
        The job identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/transcode',
            payload=job_content,
        )

    if 'job_id' in json_response:
        return json_response['job_id']

async def get_results(
        access_token: str,
        job_id: str,
    ) -> TranscodeJob:
    r"""
    Gets Transcode Results.

    For a given job_id, this method will check if the transcoding task has completed and return transcoding results.

    See: https://docs.dolby.io/media-apis/reference/media-transcode-get

    Args:
        access_token: Access token to use for authentication.
        job_id: The job identifier.

    Returns:
        An :class:`TranscodeJob` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/transcode',
            params=params
        )

    return TranscodeJob(job_id, json_response)



"""
dolbyio_rest_apis.media.analyze_speech
~~~~~~~~~~~~~~~

This module contains the functions to work with the Speech Analytics APIs.
"""

from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.analyze_speech_response import AnalyzeSpeechJob

async def start(
        access_token: str,
        job_content: str,
    ) -> str or None:
    r"""
    Starts analyzing to learn about speech in your media.

    The `input` location of your source media file and `output` location of your Analyze JSON results file are required.

    This is an asynchronous operation so you will receive a job identifier to be used to get the job status and result.

    See: https://docs.dolby.io/media-apis/reference/media-analyze-speech-post

    Beta API
    This API is being made available as an early preview.
    If you have feedback on how you'd like to use the API please reach out to share your feedback with our team.
    https://dolby.io/contact

    Args:
        access_token: Access token to use for authentication.
        job_content: Content of the job description as a JSON payload.
            You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-analyze-speech-post

    Returns:
        The job identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    print('''Beta API
    This API is being made available as an early preview.
    If you have feedback on how you\'d like to use the API please reach out to share your feedback with our team.
    https://dolby.io/contact''')

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/analyze/speech',
            payload=job_content,
        )

    if 'job_id' in json_response:
        return json_response['job_id']

async def get_results(
        access_token: str,
        job_id: str,
    ) -> AnalyzeSpeechJob:
    r"""
    Gets Speech Analytics Status.

    For a given job_id, this method will check if the processing task has completed.

    See: https://docs.dolby.io/media-apis/reference/media-analyze-speech-get

    Args:
        access_token: Access token to use for authentication.
        job_id: The job identifier.

    Returns:
        A :class:`AnalyzeSpeechJob` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/analyze/speech',
            params=params
        )

    return AnalyzeSpeechJob(job_id, json_response)



"""
Command: Version
"""

import argparse
import importlib

def add_arguments(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
		'-v', '--version',
		help='Get the version of the package',
		action='store_true',
		dest='version',
		default=False
	)

def execute_command() -> None:
    package_name = 'dolbyio_rest_apis_cli'
    package_version = importlib.metadata.version(package_name)

    print(f'Dolby.io REST APIs CLI version: {package_version}')



"""
Command: Authentication
"""

from argparse import _SubParsersAction, Namespace
from dolbyio_rest_apis.communications import authentication
import json

def command_name() -> str:
    return 'auth'

def add_arguments(sub_parsers: _SubParsersAction) -> None:
    parser = sub_parsers.add_parser(command_name(), help='Authentication')

    parser.add_argument(
		'-k', '--app_key',
		help='Your Dolby.io App Key',
		default='',
		type=str
	)

    parser.add_argument(
		'-s', '--app_secret',
		help='Your Dolby.io App Secret',
		default='',
		type=str
	)

    parser.add_argument(
		'-e', '--expires_in',
		help='''
        (Optional) Access token expiration time in seconds.
        The maximum value is 2,592,000, indicating 30 days. If no value is specified,
        the default is 600, indicating ten minutes.
        ''',
		default=None,
		type=int
	)

    parser.add_argument(
		'--api',
		help='Request an API Token otherwise a Client Access Token will be returned.',
		action='store_true',
		dest='api',
		default=False
	)

    parser.add_argument(
		'-o', '--output',
		help='Set the output format.',
		dest='output_format',
		default='json',
        choices=[ 'json', 'text', 'access_token' ]
	)

async def execute_command(args: Namespace) -> None:
    app_key = args.app_key
    app_secret = args.app_secret
    expires_in = args.expires_in
    api = args.api
    output_format = args.output_format

    if api:
        access_token = await authentication.get_api_token(
            app_key=app_key,
            app_secret=app_secret,
            expires_in=expires_in
        )
    else:
        access_token = await authentication.get_client_access_token(
            app_key=app_key,
            app_secret=app_secret,
            expires_in=expires_in
        )

    if output_format == 'json':
        print(json.dumps(access_token, indent=4))
    elif output_format == 'access_token':
        print(access_token.access_token)
    else:
        for key in access_token.keys():
            print(f'{key}: {access_token[key]}')



"""
dolbyio_rest_apis.media.analyze
~~~~~~~~~~~~~~~

This module contains the functions to work with the Analyze APIs.
"""

from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.analyze_response import AnalyzeJobResponse

async def start(
        access_token: str,
        job_content: str,
    ) -> str or None:
    r"""
    Starts analyzing to learn about your media.

    The `input` location of your source media file and `output` location of your Analyze JSON results file are required.

    This is an asynchronous operation so you will receive a `job_id` to be used to get the job status and result.

    There are additional optional parameters that can be provided to identify the type of content
    and additional loudness or validation requirements.
    See the samples for examples of what requests and responses look like.

    See: https://docs.dolby.io/media-apis/reference/media-analyze-post

    Beta API

    This API is being made available as an early preview.
    If you have feedback on how you'd like to use the API please reach out to share your feedback with our team.
    https://dolby.io/contact

    Content Length

    Media content with duration less than 2 seconds will not be processed. The API will return an ERROR in this case.

    Args:
        access_token: Access token to use for authentication.
        job_content: Content of the job description as a JSON payload.
            You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-analyze-post

    Returns:
        The job identifier.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    print('''Beta API
    This API is being made available as an early preview.
    If you have feedback on how you\'d like to use the API please reach out to share your feedback with our team.
    https://dolby.io/contact''')

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/analyze',
            payload=job_content,
        )

    if 'job_id' in json_response:
        return json_response['job_id']

async def get_results(
        access_token: str,
        job_id: str,
    ) -> AnalyzeJobResponse:
    r"""
    Gets Analyze Status.

    For a given job_id, this method will check the job status.

    See: https://docs.dolby.io/media-apis/reference/media-analyze-get

    Args:
        access_token: Access token to use for authentication.
        job_id: The job identifier.

    Returns:
        A :class:`AnalyzeJobResponse` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id
    }

    async with MediaHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/analyze',
            params=params
        )

    return AnalyzeJobResponse(job_id, json_response)



"""
dolbyio_rest_apis_cli.communications.cli
~~~~~~~~~~~~~~~

This module contains the main entry point for the CLI.

"""

import argparse
import asyncio
from dolbyio_rest_apis_cli.core import version
from dolbyio_rest_apis_cli.communications.commands import authentication, streaming, remix
from dolbyio_rest_apis_cli.communications.commands.monitor import webhooks

async def main():
    parser = argparse.ArgumentParser(description='Dolby.io REST APIs CLI - Communications')
    sub_parsers = parser.add_subparsers(dest='command')

    version.add_arguments(parser)
    authentication.add_arguments(sub_parsers)
    remix.add_arguments(sub_parsers)
    streaming.add_arguments(sub_parsers)

    command_name_monitor = 'monitor'
    parser_monitor = sub_parsers.add_parser(command_name_monitor, help='Monitor')
    sub_parsers_monitor = parser_monitor.add_subparsers(dest='sub_command')
    webhooks.add_arguments(sub_parsers_monitor)

    args = parser.parse_args()

    if args.version:
        version.execute_command()
    elif args.command is None:
        parser.print_help()
    elif args.command == authentication.command_name():
        await authentication.execute_command(args)
    elif args.command == remix.command_name():
        await remix.execute_command(args)
    elif args.command == streaming.command_name():
        await streaming.execute_command(args)
    elif args.command == command_name_monitor:
        if args.sub_command == webhooks.command_name():
            await webhooks.execute_command(args)

def cli():
    asyncio.get_event_loop().run_until_complete(main())

if __name__ == '__main__':
    cli()



"""
dolbyio_rest_apis.streaming.models.core
~~~~~~~~~~~~~~~
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default

class BaseResponse(dict):
    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.status = get_value_or_default(self, 'status', None)
        self.data = get_value_or_default(self, 'data', None)

class Error(dict):
    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.message = get_value_or_default(self, 'message', None)



"""
Command: Jobs
"""

from argparse import _SubParsersAction, Namespace
from dolbyio_rest_apis.media import jobs
import json

def command_name() -> str:
    return 'jobs'

def add_arguments(sub_parsers: _SubParsersAction) -> None:
    parser = sub_parsers.add_parser(command_name(), help='Jobs')

    parser.add_argument(
		'-a', '--access_token',
		help='Your API Access Token.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--after',
		help='Query jobs that were submitted at or after the specified date and time (inclusive).',
		required=False,
		type=str
	)

    parser.add_argument(
		'--before',
		help='''
        Query jobs that were submitted at or before the specified date and time (inclusive).
        The `submitted_before` must be the same or later than `submitted_after`.
        ''',
		required=False,
		type=str
	)

    parser.add_argument(
		'--status',
		help='Query jobs that were submitted at or after the specified date and time (inclusive).',
		required=False,
		type=str
	)

    parser.add_argument(
		'-o', '--output',
		help='Set the output format.',
		dest='output_format',
		default='json',
        choices=[ 'json', 'text' ]
	)

async def execute_command(args: Namespace) -> None:
    access_token = args.access_token
    after = args.after
    before = args.before
    status = args.status

    all_jobs = await jobs.list_all_jobs(
        access_token=access_token,
        submitted_after=after,
        submitted_before=before,
        status=status,
    )

    if args.output_format == 'json':
        print(json.dumps(all_jobs, indent=4))
    else:
        job_id = 0
        for job in all_jobs:
            job_id += 1
            print(f'Job #{job_id}/{len(all_jobs)}:')
            for key in job.keys():
                print(f' {key}: {job[key]}')
            print()



"""
dolbyio_rest_apis.streaming.models.cluster
~~~~~~~~~~~~~~~

This module contains the models used by the Cluster model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default

class Cluster(dict):
    """The :class:`Cluster` object, which represents a cluster."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.id = get_value_or_default(self, 'id', None)
        self.name = get_value_or_default(self, 'name', None)
        self.rtmp = get_value_or_default(self, 'rtmp', None)

class ClusterResponse(dict):
    """The :class:`ClusterResponse` object, which represents a cluster response."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.default_cluster = get_value_or_default(self, 'defaultCluster', None)
        self.available_clusters = []
        for cluster in self['availableClusters']:
            self.available_clusters.append(Cluster(cluster))



"""
dolbyio_rest_apis.media.models.jobs_response
~~~~~~~~~~~~~~~

This module contains the Jobs Response model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default, in_and_not_none
from .paged_response import PagedResponse

class JobsResponse(PagedResponse):
    r"""Representation of a jobs response."""

    def __init__(self, dictionary: dict):
        PagedResponse.__init__(self, dictionary)

        self.jobs = []
        if in_and_not_none(self, 'jobs'):
            for job in self['jobs']:
                self.jobs.append(Job(job))

class Job(dict):
    r"""Representation of a job."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.job_id = get_value_or_default(self, 'job_id', None)
        self.api_version = get_value_or_default(self, 'api_version', None)
        self.path = get_value_or_default(self, 'path', None)
        self.status = get_value_or_default(self, 'status', None)
        self.progress = get_value_or_default(self, 'progress', 0)
        self.duration = get_value_or_default(self, 'duration', 0.0)
        self.time_submitted = get_value_or_default(self, 'time_submitted', None)
        self.time_started = get_value_or_default(self, 'time_started', None)
        self.time_completed = get_value_or_default(self, 'time_completed', None)
        self.expiry = get_value_or_default(self, 'expiry', None)



"""
Command: Input / Output
"""

from argparse import _SubParsersAction, Namespace
from dolbyio_rest_apis.media import io

def command_name() -> str:
    return 'io'

def add_arguments(sub_parsers: _SubParsersAction) -> None:
    parser = sub_parsers.add_parser(command_name(), help='Input / Output')

    parser.add_argument(
		'-a', '--access_token',
		help='Your API Access Token.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--dlb_url',
		help='''
        The `url` should be in the form `dlb://object-key` where the object-key can be any alpha-numeric string.
        The object-key is unique to your account API Key so there is no risk of collision with other users.
        ''',
		required=True,
		type=str
	)

    parser.add_argument(
		'--file',
		help='File to upload.',
		required=True,
		type=str
	)

    sub_parsers_io = parser.add_subparsers(dest='sub_command')
    sub_parsers_io.add_parser('upload', help='Upload a file')
    sub_parsers_io.add_parser('download', help='Download a file')

async def execute_command(args: Namespace) -> None:
    access_token = args.access_token
    dlb_url = args.dlb_url
    file = args.file

    if args.sub_command == 'upload':
        upload_url = await io.get_upload_url(
            access_token=access_token,
            dlb_url=dlb_url
        )

        print(f'Upload URL: {upload_url}')

        await io.upload_file(
            upload_url=upload_url,
            file_path=file
        )

        print(f'File uploaded to {dlb_url}')
    elif args.sub_command == 'download':
        await io.download_file(
            access_token=access_token,
            dlb_url=dlb_url,
            file_path=file
        )

        print(f'File saved at {file}')



"""
dolbyio_rest_apis.communications.models
~~~~~~~~~~~~~~~

This module contains the models used by the Dolby.io APIs.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default

class AccessToken(dict):
    """The :class:`AccessToken` object, which represents the access token."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.token_type = get_value_or_default(self, 'token_type', None)
        self.access_token = get_value_or_default(self, 'access_token', None)
        self.refresh_token = get_value_or_default(self, 'refresh_token', None)
        self.expires_in_val = get_value_or_default(self, 'expires_in', 0)



"""
Command: Monitor / Webhooks
"""

from argparse import _SubParsersAction, Namespace
from dolbyio_rest_apis.communications.monitor import webhooks
import json

def command_name() -> str:
    return 'webhooks'

def add_arguments(sub_parsers: _SubParsersAction) -> None:
    parser = sub_parsers.add_parser(command_name(), help='Webhooks')

    parser.add_argument(
		'-a', '--access_token',
		help='Your API Access Token.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--cid',
		help='Identifier of the conference.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--from',
		help='The beginning of the time range (in milliseconds that have elapsed since epoch).',
		required=False,
		dest='tr_from',
		type=int,
        default=0
	)

    parser.add_argument(
		'--to',
		help='The end of the time range (in milliseconds that have elapsed since epoch).',
		required=False,
		dest='tr_to',
		type=int,
        default=9999999999999
	)

    parser.add_argument(
		'--filter',
		help='''
            The Webhook event type or an expression of its type (for example `Recording.Live.InProgress` or `Rec.*`).
            The default value of the type parameter returns all types of Webhooks.
            ''',
		required=False,
		type=str,
        default=None
	)

    parser.add_argument(
		'-o', '--output',
		help='Set the output format.',
		dest='output_format',
		default='json',
        choices=[ 'json', 'text' ]
	)

async def execute_command(args: Namespace) -> None:
    access_token = args.access_token
    cid = args.cid
    tr_from = args.tr_from
    tr_to = args.tr_to
    filter_type = args.filter
    output_format = args.output_format

    events = await webhooks.get_all_events(
        access_token=access_token,
        conference_id=cid,
        tr_from=tr_from,
        tr_to=tr_to,
        filter_type=filter_type
    )

    if output_format == 'json':
        print(json.dumps(events, indent=4))
    else:
        ev_id = 0
        for event in events:
            ev_id += 1
            print(f'Webhook event #{ev_id}/{len(events)}:')
            for key in event.keys():
                print(f' {key}: {event[key]}')
            print()



"""
dolbyio_rest_apis.communications.internal.helpers
~~~~~~~~~~~~~~~

This module contains internal helpers.
"""

from aiohttp import BasicAuth, ClientResponse, ContentTypeError
from dolbyio_rest_apis.core.http_context import HttpContext
from dolbyio_rest_apis.core.helpers import get_value_or_default
from dolbyio_rest_apis.core.http_request_error import HttpRequestError
import json
import logging
from typing import Any, Dict, List

class CommunicationsHttpContext(HttpContext):
    """HTTP Context class for Communications APIs"""

    def __init__(self):
        super().__init__()

        self._logger = logging.getLogger(CommunicationsHttpContext.__name__)

    async def _requests_post_put(
            self,
            access_token: str,
            url: str,
            method: str,
            payload: Any=None,
        ) -> Any or None:
        r"""
        Sends a POST or PUT request.

        Args:
            access_token: The Access Token to use for authentication.
            url: Where to send the request to.
            method: HTTP method, POST or PUT.
            payload: (Optional) Content of the request.

        Returns:
            The JSON response if any or None.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {access_token}',
        }

        if payload is None:
            payload = '{}' # The REST APIs don't support an empty payload
        else:
            payload = json.dumps(payload, indent=4)

        return await self._send_request(
            method=method,
            url=url,
            headers=headers,
            data=payload,
        )

    async def requests_put(
            self,
            access_token: str,
            url: str,
            payload: Any=None,
        ) -> Any or None:
        r"""
        Sends a PUT request.

        Args:
            access_token: The Access Token to use for authentication.
            url: Where to send the request to.
            payload: (Optional) Content of the request.

        Returns:
            The JSON response if any or None.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        return await self._requests_post_put(
            access_token=access_token,
            url=url,
            method='PUT',
            payload=payload
        )

    async def requests_post(
            self,
            access_token: str,
            url: str,
            payload: Any=None,
        ) -> Any or None:
        r"""
        Sends a POST request.

        Args:
            access_token: The Access Token to use for authentication.
            url: Where to send the request to.
            payload: (Optional) Content of the request.

        Returns:
            The JSON response if any or None.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        return await self._requests_post_put(
            access_token=access_token,
            url=url,
            method='POST',
            payload=payload
        )

    async def requests_post_basic_auth(
            self,
            app_key: str,
            app_secret: str,
            url: str,
            json_payload: str=None,
            data: Dict[str, Any]=None,
        ) -> Any or None:
        r"""
        Sends a POST request with Basic authentication.

        Args:
            app_key: The Dolby.io App Key.
            app_secret: The Dolby.io App Secret.
            url: Where to send the request to.
            json_payload: (Optional) Content of the request as JSON payload.
            data: (Optional) Content of the request.

        Returns:
            The JSON response if any or None.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """
        if data is None:
            content_type = 'application/json'
            if json_payload is None:
                payload = '{}' # The REST APIs don't support an empty payload
            else:
                payload = json.dumps(json_payload, indent=4)
        else:
            content_type = 'application/x-www-form-urlencoded'
            payload = data

        headers = {
            'Accept': 'application/json',
            'Content-Type': content_type,
        }

        return await self._send_request(
            method='POST',
            url=url,
            headers=headers,
            auth=BasicAuth(app_key, app_secret),
            data=payload,
        )

    async def requests_delete(
            self,
            access_token: str,
            url: str,
        ) -> None:
        r"""
        Sends a DELETE request.

        Args:
            access_token: The Access Token to use for authentication.
            url: Where to send the request to.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        headers = {
            'Accept': 'application/json',
            'Authorization': f'Bearer {access_token}',
        }

        return await self._send_request(
            method='DELETE',
            url=url,
            headers=headers,
        )

    async def requests_get(
            self,
            access_token: str,
            url: str,
            params: Dict[str, Any]=None,
        ) -> Any or None:
        r"""
        Sends a GET request.

        Args:
            access_token: The Access Token to use for authentication.
            url: Where to send the request to.
            params: (Optional) URL query parameters.

        Returns:
            The JSON response if any or None.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        headers = {
            'Accept': 'application/json',
            'Authorization': f'Bearer {access_token}',
        }

        return await self._send_request(
            method='GET',
            url=url,
            params=params,
            headers=headers,
        )

    async def requests_get_all(
            self,
            access_token: str,
            url: str,
            property_name: str,
            params: Dict[str, Any]=None,
            page_size: int=100,
        ) -> List[Any]:
        r"""
        Sends a GET request and returns all elements from all pages.

        Args:
            access_token: The Access Token to use for authentication.
            url: Where to send the request to.
            property_name: Name of the property that contains the elements of the page
            params: (Optional) URL query parameters.
            page_size: (Optional) number of elements per page.

        Returns:
            The list of elements from all pages.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """
        if params is None:
            params = {}

        elements: List[Any] = []

        while True:
            json_response = await self.requests_get(
                access_token=access_token,
                url=url,
                params=params,
            )

            if property_name in json_response:
                sub_result = json_response[property_name]
                for element in sub_result:
                    elements.append(element)

                if len(sub_result) < page_size:
                    break

            if not 'next' in json_response:
                break

            params['start'] = json_response['next']
            if params['start'] is None or params['start'] == '':
                break

        return elements

    async def requests_get_basic_auth(
            self,
            app_key: str,
            app_secret: str,
            url: str,
        ) -> Any or None:
        r"""
        Sends a GET request with Basic authentication.

        Args:
            app_key: The Dolby.io App Key.
            app_secret: The Dolby.io App Secret.
            url: Where to send the request to.

        Returns:
            The JSON response if any or None.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
        }

        return await self._send_request(
            method='GET',
            url=url,
            headers=headers,
            auth=BasicAuth(app_key, app_secret),
        )

    async def download(
            self,
            access_token: str,
            url: str,
            accept: str,
            file_path: str,
        ) -> None:
        r"""
        Downloads a file.

        Args:
            access_token: The Access Token to use for authentication.
            url: Where to send the request to.
            accept: Accept HTTP header.
            file_path: Where to save the file.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """
        headers = {
            'Accept': accept,
            'Authorization': f'Bearer {access_token}',
        }

        await self._download_file(
            url=url,
            headers=headers,
            file_path=file_path
        )

    async def _raise_for_status(self, http_response: ClientResponse):
        r"""Raises :class:`HttpRequestError` or :class:`ClientResponseError`, if one occurred."""

        if 400 <= http_response.status < 500:
            if 400 < http_response.status < 404:
                self._logger.error('Unauthorized to get data from the url %s - Response code %i', http_response.url, http_response.status)
            elif http_response.status == 404:
                self._logger.error('Unable to get data from the url %s - Not found (404)', http_response.url)
            else:
                self._logger.error('Did not find data at the url %s - Response code %i', http_response.url, http_response.status)

            try:
                json_response = await http_response.json()

                error_type = get_value_or_default(json_response, 'type', None)
                error_code = get_value_or_default(json_response, 'error_code', 0)
                if error_code == 0:
                    error_code = get_value_or_default(json_response, 'status', 0)
                error_reason = get_value_or_default(json_response, 'error_reason', None)
                if error_reason is None:
                    error_reason = get_value_or_default(json_response, 'error', None)
                error_description = get_value_or_default(json_response, 'error_description', None)
                if error_description is None:
                    error_description = get_value_or_default(json_response, 'message', None)

                raise HttpRequestError(http_response, error_type, error_code, error_reason, error_description)
            except (ValueError, ContentTypeError): # If the response body does not contain valid json.
                pass

        http_response.raise_for_status()



"""
dolbyio_rest_apis.communications.conference
~~~~~~~~~~~~~~~

This module contains the functions to work with the conference API.
"""

from dataclasses import asdict
from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.core.helpers import get_value, add_if_not_none
from dolbyio_rest_apis.core.urls import get_comms_url_v2
from .models import UserToken, Conference, SpatialAudioEnvironment, SpatialAudioListener, SpatialAudioUser, RTCPMode, Participant, VideoCodec
from typing import List

async def create_conference(
        access_token: str,
        owner_external_id: str,
        alias: str=None,
        pincode: str=None,
        dolby_voice: bool=True,
        live_recording: bool=False,
        rtcp_mode: RTCPMode=RTCPMode.AVERAGE,
        ttl: int=None,
        video_codec: VideoCodec=None,
        audio_only: bool=False,
        participants: List[Participant]=None,
        recording_formats: List[str]=None,
        region: str=None,
    ) -> Conference:
    r"""
    Creates a conference.

    See: https://docs.dolby.io/communications-apis/reference/create-conference

    Args:
        access_token: Access token to use for authentication.
        owner_external_id: External ID of the conference owner.
        alias: (Optional) Name of the conference.
        pincode: (Optional) The PIN code of the conference.
            This applies to conferences using PSTN (telephony network).
        dolby_voice: (Optional) Indicates if Dolby Voice is enabled for the conference.
            The `True` value creates the conference with Dolby Voice enabled.
        live_recording: (Optional) Indicates if live recording is enabled for the conference.
        rtcp_mode: (Optional) Specifies the bitrate adaptation mode for the video transmission.
        ttl: (Optional) Specifies the time to live that enables customizing the waiting time
            (in seconds) and terminating empty conferences.
        audio_only: (Optional) If `True`, the conference does not allow participants to enable video.
        video_codec: (Optional) Specifies the video codec (VP8 or H264) for the conference.
        participants: List of the :class:`Participant` object to update the permissions.
        recording_formats: If specified, the default RecordingConfiguration is overridden.
            Specifies the recording format. Valid values are 'mp3' and 'mp4'.
        region: Dolby.io region where you want the conference to be hosted. Can be one of:
            - au: Australia
            - ca: Canada
            - eu: Europe
            - in: India
            - us: United States

    Returns:
        A :class:`Conference` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    parameters = {
        'dolbyVoice': dolby_voice,
        'liveRecording': live_recording,
        'rtcpMode': rtcp_mode.value,
    }
    add_if_not_none(parameters, 'pincode', pincode)
    add_if_not_none(parameters, 'ttl', ttl)
    add_if_not_none(parameters, 'audioOnly', audio_only)
    add_if_not_none(parameters, 'videoCodec', video_codec)

    if recording_formats is not None and len(recording_formats) > 0:
        parameters['recording'] = {
            'format': recording_formats
        }

    payload = {
        'ownerExternalId': owner_external_id,
        'parameters': parameters,
    }
    add_if_not_none(payload, 'alias', alias)

    if participants is not None and len(participants) > 0:
        obj_participants = { }

        for participant in participants:
            if isinstance(participant, Participant):
                permissions = []
                for p in participant.permissions:
                    permissions.append(get_value(p))

                obj_participants[participant.external_id] = {
                    'permissions': permissions,
                    'notification': participant.notify,
                }

        payload['participants'] = obj_participants

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2(region)}/conferences/create',
            payload=payload
        )

    return Conference(json_response)

async def invite(
        access_token: str,
        conference_id: str,
        participants: List[Participant],
    ) -> List[UserToken]:
    r"""
    Invites participants to an ongoing conference. This API can also be used to generate new conference access tokens
    for an ongoing conference. If the invite request includes participants that are already in the conference, a new
    conference access token is not generated and an invitation is not sent.

    See: https://docs.dolby.io/communications-apis/reference/invite-to-conference

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        participants: List of the :class:`Participant` object to invite to the conference.

    Returns:
        A list of :class:`UserToken` objects.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    obj_participants = { }
    for participant in participants:
        if isinstance(participant, Participant):
            permissions = []
            for p in participant.permissions:
                permissions.append(get_value(p))

            obj_participants[participant.external_id] = {
                'permissions': permissions,
                'notification': participant.notify,
            }

    payload = {
        'participants': obj_participants,
    }

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/{conference_id}/invite',
            payload=payload
        )

    user_tokens = []
    for key in json_response.keys():
        user_token = UserToken(key, json_response[key])
        user_tokens.append(user_token)

    return user_tokens

async def kick(
        access_token: str,
        conference_id: str,
        external_ids: List[str],
    ) -> None:
    r"""
    Kicks participants from an ongoing conference.

    See: https://docs.dolby.io/communications-apis/reference/kick-from-conference

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        external_ids: List external IDs of the participants to kick out of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    payload = {
        'externalIds': external_ids,
    }

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/{conference_id}/kick',
            payload=payload
        )

async def send_message(
        access_token: str,
        conference_id: str,
        from_external_id: str,
        to_external_ids: List[str],
        message: str
    ) -> None:
    r"""
    Sends a message to some or all participants in a conference.

    See: https://docs.dolby.io/communications-apis/reference/send-message

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        from_external_id: The external ID of the author of the message.
        to_external_ids: A list of external IDs that will receive the message.
            If empty, the message will be broadcasted to all participants in the conference.
        message: The message to send.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    payload = {
        'from': from_external_id,
        'message': message,
    }

    if to_external_ids is not None and len(to_external_ids) > 0:
        payload['to'] = to_external_ids

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/{conference_id}/message',
            payload=payload
        )

async def set_spatial_listeners_audio(
        access_token: str,
        conference_id: str,
        environment: SpatialAudioEnvironment,
        listener: SpatialAudioListener,
        users: List[SpatialAudioUser],
    ) -> None:
    r"""
    Sets the spatial audio scene for all listeners in an ongoing conference.
    This sets the spatial audio environment, the position and direction for all listeners with the spatialAudio flag enabled.
    The calls are not cumulative, and each call sets all the spatial listener values.
    Participants who do not have a position set are muted.

    See: https://docs.dolby.io/communications-apis/reference/set-spatial-listeners-audio

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        environment: The spatial environment of an application,
            so the audio renderer understands which directions the application considers
            forward, up, and right and which units it uses for distance.
        listener: The listener's audio position and direction, defined using Cartesian coordinates.
        users: The users' audio positions, defined using Cartesian coordinates.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    obj_users = { }
    for user in users:
        obj_users[user.external_id] = asdict(user.position)

    payload = {
        'environment': asdict(environment),
        'listener': asdict(listener),
        'users': obj_users,
    }

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_put(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/{conference_id}/spatial-listeners-audio',
            payload=payload
        )

async def update_permissions(
        access_token: str,
        conference_id: str,
        participants: List[Participant],
    ) -> List[UserToken]:
    r"""
    Update permissions for participants in a conference. When a participant's permissions are updated, the new token
    is sent directly to the SDK. The SDK automatically receives, stores, and manages the new token
    and a permissionsUpdated event is sent.

    See: https://docs.dolby.io/communications-apis/reference/update-permissions

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        participants: List of the :class:`Participant` object to update the permissions.

    Returns:
        A list of :class:`UserToken` objects.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    obj_participants = { }
    for participant in participants:
        if isinstance(participant, Participant):
            permissions = []
            for p in participant.permissions:
                permissions.append(get_value(p))

            obj_participants[participant.external_id] = {
                'permissions': permissions,
            }

    payload = {
        'participants': obj_participants,
    }

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/{conference_id}/permissions',
            payload=payload
        )

    user_tokens = []
    for key in json_response.keys():
        user_token = UserToken(key, json_response[key])
        user_tokens.append(user_token)

    return user_tokens

async def terminate(
        access_token: str,
        conference_id: str,
    ) -> None:
    r"""
    Terminates an ongoing conference and removes all remaining participants from the conference.

    See: https://docs.dolby.io/communications-apis/reference/terminate-conference

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_delete(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/{conference_id}',
        )



"""
dolbyio_rest_apis.media.models.paged_response
~~~~~~~~~~~~~~~

This module contains the Paged Response base model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default

class PagedResponse(dict):
    r"""Representation of a paged response."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.next_token = get_value_or_default(self, 'next_token', None)
        self.count = get_value_or_default(self, 'count', None)



"""
dolbyio_rest_apis.communications.monitor.recordings
~~~~~~~~~~~~~~~

This module contains the functions to work with the monitor API related to recordings.
"""

from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.communications.monitor.models import GetRecordingsResponse, Recording, DolbyVoiceRecording
from dolbyio_rest_apis.core.urls import get_comms_monitor_url
from typing import Any, List

async def get_recordings(
        access_token: str,
        tr_from: int=0,
        tr_to: int=9999999999999,
        maximum: int=100,
        start: str=None,
        perm: bool=False,
    ) -> GetRecordingsResponse:
    r"""
    Get recording details

    Get a list of the recorded conference metadata, such as duration or size of the recording.
    This API checks only the recordings that have ended during a specific time range.
    Recordings are indexed based on the ending time.

    See: https://docs.dolby.io/communications-apis/reference/get-recordings

    Args:
        access_token: Access token to use for authentication.
        tr_from: The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: The end of the time range (in milliseconds that have elapsed since epoch).
        maximum: The maximum number of displayed results.
            We recommend setting the proper value of this parameter to shorten the response time.
        start: When the results span multiple pages, use this option to navigate through pages.
            By default, only the max number of results is displayed. To see the next results,
            set the start parameter to the value of the next key returned in the previous response.
        perm: When set to true, the URL is replaced with a monitor API signed URL with unlimited validity.

    Returns:
        A :class:`GetRecordingsResponse` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/recordings'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': maximum,
        'perm': str(perm).lower(),
    }

    if not start is None:
        params['start'] = start

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
            params=params,
        )

    return GetRecordingsResponse(json_response)

async def get_all_recordings(
        access_token: str,
        tr_from: int=0,
        tr_to: int=9999999999999,
        page_size: int=100,
        perm: bool=False,
    ) -> List[Recording]:
    r"""
    Get all recording details

    Get a list of all the recorded conference metadata, such as duration or size of the recording.
    This API checks only the recordings that have ended during a specific time range.
    Recordings are indexed based on the ending time.

    See: https://docs.dolby.io/communications-apis/reference/get-recordings

    Args:
        access_token: Access token to use for authentication.
        tr_from: The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: The end of the time range (in milliseconds that have elapsed since epoch).
        page_size: (Optional) Number of elements to return per page.
        perm: When set to true, the URL is replaced with a monitor API signed URL with unlimited validity.

    Returns:
        A list of :class:`Recording` objects.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/recordings'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': page_size,
        'perm': str(perm).lower(),
    }

    recordings = []

    async with CommunicationsHttpContext() as http_context:
        elements: List[Any] = await http_context.requests_get_all(
            access_token=access_token,
            url=url,
            params=params,
            property_name='recordings',
            page_size=page_size
        )

    recordings: List[Recording] = []
    for element in elements:
        recording = Recording(element)
        recordings.append(recording)

    return recordings

async def get_recording(
        access_token: str,
        conference_id: str,
        tr_from: int=0,
        tr_to: int=9999999999999,
        page_size: int=100,
        perm: bool=False,
    ) -> GetRecordingsResponse:
    r"""
    Get the recording of a specific conference

    Get a list of the recorded conference metadata, such as duration or size of the recording.
    This API checks the recordings that have ended during a specific time range.
    Recordings are indexed based on the ending time.

    See: https://docs.dolby.io/communications-apis/reference/get-conference-recordings

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        tr_from: The beginning of the time range (in milliseconds that have elapsed since epoch).
        tr_to: The end of the time range (in milliseconds that have elapsed since epoch).
        page_size: (Optional) Number of elements to return per page.
        perm: When set to true, the URL is replaced with a monitor API signed URL with unlimited validity.

    Returns:
        A :class:`GetRecordingsResponse` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_monitor_url()}/conferences/{conference_id}/recordings'

    params = {
        'from': tr_from,
        'to': tr_to,
        'max': page_size,
        'perm': str(perm).lower(),
    }

    recordings = []

    async with CommunicationsHttpContext() as http_context:
        elements: List[Any] = await http_context.requests_get_all(
            access_token=access_token,
            url=url,
            params=params,
            property_name='recordings',
            page_size=page_size
        )

    recordings: List[Recording] = []
    for element in elements:
        recording = Recording(element)
        recordings.append(recording)

    return recordings

async def delete_recording(
        access_token: str,
        conference_id: str,
    ) -> None:
    r"""
    Delete recordings

    Delete all recording data related to a specific conference.

    Warning: After deleting the recording, it is not possible to restore the recording data.

    See: https://docs.dolby.io/communications-apis/reference/delete-conference-recordings

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    url = f'{get_comms_monitor_url()}/conferences/{conference_id}/recordings'

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_delete(
            access_token=access_token,
            url=url,
        )

async def get_dolby_voice_recordings(
        access_token: str,
        conference_id: str,
    ) -> DolbyVoiceRecording:
    r"""
    Get Dolby Voice audio recordings of a conference

    Get details of all Dolby Voice-based audio recordings, and associated split recordings,
    for a given conference and download the conference recording in the MP3 audio format.

    See: https://docs.dolby.io/communications-apis/reference/get-dolby-voice-audio-recordings

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Returns:
        A :class:`DolbyVoiceRecording` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    url = f'{get_comms_monitor_url()}/conferences/{conference_id}/recordings/audio'

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_get(
            access_token=access_token,
            url=url,
        )

    return DolbyVoiceRecording(json_response)



"""
dolbyio_rest_apis.communications.recording
~~~~~~~~~~~~~~~

This module contains the functions to work with the recording API.
"""

from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.core.urls import get_comms_url_v2

async def start(
        access_token: str,
        conference_id: str,
        layout_url: str=None,
    ) -> None:
    r"""
    Starts recording for the specified conference.
    You can specify a custom layout URL per recording request.
    The `layoutURL` parameter overrides the layout URL configured in the dashboard.

    See: https://docs.dolby.io/communications-apis/reference/api-recording-start

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        layout_url: Overwrites the layout URL configuration.
            This field is ignored if it is not relevant regarding recording configuration,
            for example if live_recording set to false or if the recording is MP3 only.
            - `null`: uses the layout URL configured in the dashboard (if no URL is set in the dashboard, then uses the Dolby.io default);
            - `default`: uses the Dolby.io default layout;
            - URL string: uses this layout URL

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_url_v2()}/conferences/mix/{conference_id}/recording/start'

    payload = {}
    add_if_not_none(payload, 'layoutUrl', layout_url)

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=url,
            payload=payload,
        )

async def stop(
        access_token: str,
        conference_id: str,
    ) -> None:
    r"""
    Stops the recording of the specified conference.

    See: https://docs.dolby.io/communications-apis/reference/api-recording-stop

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    url = f'{get_comms_url_v2()}/conferences/mix/{conference_id}/recording/stop'

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=url,
        )



"""
dolbyio_rest_apis.core.urls
~~~~~~~~~~~~~~~

This module contains the URLs to use to call the REST APIs.
"""

API_URL = 'api.dolby.io'

COMMS_URL = 'comms.api.dolby.io'
COMMS_SESSION_URL = 'session.voxeet.com/v1'

SAPI_URL = 'api.millicast.com'

MAPI_URL = 'api.dolby.com'

def get_api_url() -> str:
    return f'https://{API_URL}/v1'

def get_comms_monitor_url() -> str:
    return f'https://{COMMS_URL}/v1/monitor'

def get_comms_url_v2(region: str = None) -> str:
    if region is None:
        return f'https://{COMMS_URL}/v2'
    return f'https://{region}.{COMMS_URL}/v2'

def get_comms_session_url() -> str:
    return f'https://{COMMS_SESSION_URL}'

def get_rts_url() -> str:
    return f'https://{SAPI_URL}'

def get_mapi_url() -> str:
    return f'https://{MAPI_URL}'



"""
dolbyio_rest_apis.streaming.internal.helpers
~~~~~~~~~~~~~~~

This module contains internal helpers.
"""

from aiohttp import ClientResponse, ContentTypeError
from dolbyio_rest_apis.core.http_context import HttpContext
from dolbyio_rest_apis.core.http_request_error import HttpRequestError
from dolbyio_rest_apis.streaming.models.core import BaseResponse, Error
import json
import logging
from typing import Any, Mapping

class StreamingHttpContext(HttpContext):
    """HTTP Context class for Real-time Streaming APIs"""

    def __init__(self):
        super().__init__()

        self._logger = logging.getLogger(StreamingHttpContext.__name__)

    async def _requests_post_put(
            self,
            api_secret: str,
            url: str,
            method: str,
            payload: Any=None,
            params: Mapping[str, str]=None,
        ) -> dict:
        r"""
        Sends a POST or PUT request.

        Args:
            api_secret: API secret to use for authentication.
            url: Where to send the request to.
            method: HTTP method, POST or PUT.
            payload: (Optional) Content of the request.
            params: (Optional) URL query parameters.

        Returns:
            The JSON response.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {api_secret}',
        }

        if not isinstance(payload, str):
            payload = json.dumps(payload, indent=4)

        json_response = await self._send_request(
            method=method,
            url=url,
            params=params,
            headers=headers,
            data=payload,
        )

        base = BaseResponse(json_response)
        return base.data

    async def requests_post(
            self,
            api_secret: str,
            url: str,
            payload: Any=None,
            params: Mapping[str, str]=None,
        ) -> dict:
        r"""
        Sends a POST request.

        Args:
            api_secret: API secret to use for authentication.
            url: Where to send the request to.
            payload: (Optional) Content of the request.
            params: (Optional) URL query parameters.

        Returns:
            The JSON response.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        return await self._requests_post_put(
            api_secret=api_secret,
            url=url,
            method='POST',
            payload=payload,
            params=params,
        )

    async def requests_put(
            self,
            api_secret: str,
            url: str,
            payload: Any=None,
            params: Mapping[str, str]=None,
        ) -> dict:
        r"""
        Sends a PUT request.

        Args:
            api_secret: API secret to use for authentication.
            url: Where to send the request to.
            payload: (Optional) Content of the request.
            params: (Optional) URL query parameters.

        Returns:
            The JSON response.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        return await self._requests_post_put(
            api_secret=api_secret,
            url=url,
            method='PUT',
            payload=payload,
            params=params,
        )

    async def requests_get(
            self,
            api_secret: str,
            url: str,
            params: Mapping[str, str]=None,
        ) -> dict:
        r"""
        Sends a GET request.

        Args:
            api_secret: API secret to use for authentication.
            url: Where to send the request to.
            params: (Optional) URL query parameters.

        Returns:
            The JSON response.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        headers = {
            'Accept': 'application/json',
            'Authorization': f'Bearer {api_secret}',
        }

        json_response = await self._send_request(
            method='GET',
            url=url,
            params=params,
            headers=headers,
        )

        base = BaseResponse(json_response)
        return base.data

    async def requests_delete(
            self,
            access_token: str,
            url: str,
            params: Mapping[str, str]=None,
        ) -> None:
        r"""
        Sends a DELETE request.

        Args:
            access_token: Access token to use for authentication.
            url: Where to send the request to.
            params: (Optional) URL query parameters.

        Raises:
            HttpRequestError: If a client error one occurred.
            HTTPError: If one occurred.
        """

        headers = {
            'Accept': 'application/json',
            'Authorization': f'Bearer {access_token}',
        }

        return await self._send_request(
            method='DELETE',
            url=url,
            params=params,
            headers=headers,
        )

    async def _raise_for_status(self, http_response: ClientResponse):
        r"""Raises :class:`HttpRequestError` or :class:`ClientResponseError`, if one occurred."""

        if 400 <= http_response.status < 500:
            if 400 < http_response.status < 404:
                self._logger.error('Unauthorized to get data from the url %s - Response code %i', http_response.url, http_response.status)
            elif http_response.status == 404:
                self._logger.error('Unable to get data from the url %s - Not found (404)', http_response.url)
            else:
                self._logger.error('Did not find data at the url %s - Response code %i', http_response.url, http_response.status)

            try:
                json_response = await http_response.json()
                base_response = BaseResponse(json_response)
                err = Error(base_response.data)

                raise HttpRequestError(http_response, '', http_response.status, base_response.status, err.message)
            except (ValueError, ContentTypeError): # If the response body does not contain valid json.
                pass

        http_response.raise_for_status()



"""
dolbyio_rest_apis.media.models.analyze_speech_response
~~~~~~~~~~~~~~~

This module contains the Analyze Speech model.
"""

from .job_response import JobResponse

class AnalyzeSpeechJobResult(dict):
    """The :class:`AnalyzeSpeechJobResult` object, which represents the result for an analyze speech job."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        if 'media_info' in dictionary:
            self.media_info = dictionary['media_info']
        if 'processed_region' in dictionary:
            self.media_info = dictionary['processed_region']

class AnalyzeSpeechJob(JobResponse):
    """The :class:`AnalyzeSpeechJob` object, which represents the result for an analyze speech job."""

    def __init__(self, job_id, dictionary: dict):
        JobResponse.__init__(self, job_id, dictionary)

        if 'result' in dictionary:
            self.result = AnalyzeSpeechJobResult(dictionary)



"""
dolbyio_rest_apis.streaming.models.subscribe_token
~~~~~~~~~~~~~~~

This module contains the models used by the Subscribe Token models.
"""

from typing import List
from dolbyio_rest_apis.core.helpers import get_value_or_default

class SubscribeTokenStream(dict):
    """The :class:`SubscribeTokenStream` object, which represents a Subscribe Token Stream."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.stream_name = get_value_or_default(self, 'streamName', None)
        self.is_regex = get_value_or_default(self, 'isRegex', False)

class SubscribeToken(dict):
    """The :class:`SubscribeToken` object, which represents a Subscribe Token."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.id = get_value_or_default(self, 'id', None)
        self.label = get_value_or_default(self, 'label', None)
        self.token = get_value_or_default(self, 'token', None)
        self.added_on = get_value_or_default(self, 'addedOn', None)
        self.expires_on = get_value_or_default(self, 'expiresOn', None)
        self.is_active = get_value_or_default(self, 'isActive', None)
        self.streams = []
        for stream in self['streams']:
            self.streams.append(SubscribeTokenStream(stream))
        self.allowed_origins = get_value_or_default(self, 'allowedOrigins', None)
        self.allowed_ip_addresses = get_value_or_default(self, 'allowedIpAddresses', None)
        self.bind_ips_on_usage = get_value_or_default(self, 'bindIpsOnUsage', None)
        self.allowed_countries = get_value_or_default(self, 'allowedCountries', None)
        self.denied_countries = get_value_or_default(self, 'deniedCountries', None)
        self.origin_cluster = get_value_or_default(self, 'originCluster', None)

class CreateUpdateSubscribeTokenStream():
    """The :class:`UpdateSubscribeTokenStream` object, which represents an update Subscribe Token Stream."""

    def __init__(self, stream_name: str, is_regex: bool):
        self.stream_name = stream_name
        self.is_regex = is_regex

class UpdateSubscribeToken():
    """The :class:`UpdateSubscribeToken` object, which represents an Update Subscribe Token request."""

    def __init__(self):
        self.label: str | None = None
        self.refresh_token: bool | None = None
        self.is_active: bool | None = None
        self.add_token_streams: List[CreateUpdateSubscribeTokenStream] | None = None
        self.remove_token_streams: List[CreateUpdateSubscribeTokenStream] | None = None
        self.update_allowed_origins: List[str] | None = None
        self.update_allowed_ip_addresses: List[str] | None = None
        self.update_bind_ips_on_usage: int | None = None
        self.update_allowed_countries: List[str] | None = None
        self.update_denied_countries: List[str] | None = None
        self.update_origin_cluster: str | None = None

class CreateSubscribeToken():
    """The :class:`CreateSubscribeToken` object, which represents a Create Subscribe Token request."""

    def __init__(self, label: str):
        self.label = label
        self.expires_on: str | None = None
        self.streams: List[CreateUpdateSubscribeTokenStream] = []
        self.allowed_origins: List[str] | None = None
        self.allowed_ip_addresses: List[str] | None = None
        self.bind_ips_on_usage: int | None = None
        self.allowed_countries: List[str] | None = None
        self.denied_countries: List[str] | None = None
        self.origin_cluster: str | None = None



"""
Command: Music Mastering
"""

from argparse import _SubParsersAction, Namespace
from dolbyio_rest_apis.media import mastering
import json

def command_name() -> str:
    return 'mastering'

def add_arguments(sub_parsers: _SubParsersAction) -> None:
    parser = sub_parsers.add_parser(command_name(), help='Music Mastering API')

    parser.add_argument(
		'-a', '--access_token',
		help='Your API Access Token.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--preview',
		help='Music Mastering Preview.',
		action='store_true',
		default=False
	)

    sub_parsers_io = parser.add_subparsers(dest='sub_command')

    start_parser = sub_parsers_io.add_parser('start', help='Starts mastering to improve your music.')

    start_parser.add_argument(
		'--file',
		help='''
        File that contains the job description as a JSON payload.
        You can find the definition at this URL: https://docs.dolby.io/media-apis/reference/media-music-mastering-post
        ''',
		required=True,
		type=str
	)

    result_parser = sub_parsers_io.add_parser('result', help='Gets Enhance Results')

    result_parser.add_argument(
		'-i', '--job_id',
		help='Job identifier.',
		required=True,
		type=str
	)

    result_parser.add_argument(
		'-o', '--output',
		help='Set the output format.',
		dest='output_format',
		default='json',
        choices=[ 'json', 'text' ]
	)

async def execute_command(args: Namespace) -> None:
    access_token = args.access_token

    if args.sub_command == 'start':
        file_path = args.file
        with open(file_path, 'r', encoding='UTF-8') as f:
            job_content = f.read()

        if args.preview:
            job_id = await mastering.start_preview(
                access_token=access_token,
                job_content=job_content
            )
        else:
            job_id = await mastering.start(
                access_token=access_token,
                job_content=job_content
            )

        print(job_id)
    elif args.sub_command == 'result':
        job_id = args.job_id

        if args.preview:
            job_result = await mastering.get_preview_results(
                access_token=access_token,
                job_id=job_id
            )
        else:
            job_result = await mastering.get_results(
                access_token=access_token,
                job_id=job_id
            )

        if args.output_format == 'json':
            print(json.dumps(job_result, indent=4))
        else:
            for key in job_result.keys():
                print(f'{key}: {job_result[key]}')



"""
dolbyio_rest_apis.media.jobs
~~~~~~~~~~~~~~~

This module contains the functions to work with the Jobs APIs.
"""

from typing import List
from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.core.urls import get_mapi_url
from dolbyio_rest_apis.media.internal.http_context import MediaHttpContext
from dolbyio_rest_apis.media.models.jobs_response import JobsResponse, Job

async def _list_jobs(
        http_context: MediaHttpContext,
        access_token: str,
        submitted_after: str=None,
        submitted_before: str=None,
        status: str=None,
        next_token: str=None,
    ) -> JobsResponse:
    params = { }
    add_if_not_none(params, 'submitted_after', submitted_after)
    add_if_not_none(params, 'submitted_before', submitted_before)
    add_if_not_none(params, 'status', status)
    add_if_not_none(params, 'next_token', next_token)

    json_response = await http_context.requests_get(
        access_token=access_token,
        url=f'{get_mapi_url()}/media/jobs',
        params=params
    )

    return JobsResponse(json_response)

async def list_jobs(
        access_token: str,
        submitted_after: str=None,
        submitted_before: str=None,
        status: str=None,
        next_token: str=None,
    ) -> JobsResponse:
    r"""
    Query Media Jobs.

    List of jobs previously submitted, up to the last 31 days.

    See: https://docs.dolby.io/media-apis/reference/media-jobs-get

    Args:
        access_token: Access token to use for authentication.
        submitted_after: (Optional) Query jobs that were submitted at or after the specified date and time (inclusive).
        submitted_before: (Optional) Query jobs that were submitted at or before the specified date and time (inclusive).
            The `submitted_before` must be the same or later than `submitted_after`.
        status: (Optional) Query jobs that have the specified status.
        next_token: (Optional) Used when querying the next page of jobs. Specify the `next_token` that was returned in the previous call.

    Returns:
        A :class:`JobsResponse` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    async with MediaHttpContext() as http_context:
        json_response = await _list_jobs(
            http_context=http_context,
            access_token=access_token,
            submitted_after=submitted_after,
            submitted_before=submitted_before,
            status=status,
            next_token=next_token,
        )

    return JobsResponse(json_response)

async def list_all_jobs(
        access_token: str,
        submitted_after: str=None,
        submitted_before: str=None,
        status: str=None,
    ) -> JobsResponse:
    r"""
    Query Media Jobs.

    List of all jobs previously submitted, up to the last 31 days.

    See: https://docs.dolby.io/media-apis/reference/media-jobs-get

    Args:
        access_token: Access token to use for authentication.
        submitted_after: (Optional) Query jobs that were submitted at or after the specified date and time (inclusive).
        submitted_before: (Optional) Query jobs that were submitted at or before the specified date and time (inclusive).
            The `submitted_before` must be the same or later than `submitted_after`.
        status: (Optional) Query jobs that have the specified status.

    Returns:
        A list of :class:`Job` objects.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    jobs: List[Job] = []

    async with MediaHttpContext() as http_context:
        next_token = None
        while True:
            page: JobsResponse = _list_jobs(
                http_context=http_context,
                access_token=access_token,
                submitted_after=submitted_after,
                submitted_before=submitted_before,
                status=status,
                next_token=next_token,
            )

            next_token = page.next_token
            for job in page.jobs:
                jobs.append(job)

            if page.next_token is None or page.next_token == '':
                break

    return jobs

async def cancel(
        access_token: str,
        job_id: str,
    ) -> None:
    r"""
    Requests cancellation of a previously submitted job.

    See: https://docs.dolby.io/media-apis/reference/media-jobs-cancel-post

    Args:
        access_token: Access token to use for authentication.
        job_id: Identifier of the job to cancel.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """
    params = {
        'job_id': job_id,
    }

    async with MediaHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_mapi_url()}/media/jobs/cancel',
            params=params,
        )



"""
dolbyio_rest_apis.streaming.cluster
~~~~~~~~~~~~~~~

This module contains the functions to work with the Cluster APIs.
"""

from dolbyio_rest_apis.core.urls import get_rts_url
from dolbyio_rest_apis.streaming.internal.http_context import StreamingHttpContext
from dolbyio_rest_apis.streaming.models.cluster import ClusterResponse

async def read(
        api_secret: str,
    ) -> ClusterResponse:
    async with StreamingHttpContext() as http_context:
        json_response = await http_context.requests_get(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/cluster',
        )

    return ClusterResponse(json_response)

async def update(
        api_secret: str,
        default_cluster: str,
    ) -> ClusterResponse:
    payload = {
        'defaultCluster': default_cluster,
    }

    async with StreamingHttpContext() as http_context:
        json_response = await http_context.requests_put(
            api_secret=api_secret,
            url=f'{get_rts_url()}/api/cluster',
            payload=payload,
        )

    return ClusterResponse(json_response)



"""
dolbyio_rest_apis.communications.streaming
~~~~~~~~~~~~~~~

This module contains the functions to work with the streaming API.
"""

from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.core.urls import get_comms_url_v2

async def start_rtmp(
        access_token: str,
        conference_id: str,
        rtmp_url: str,
        layout_url: str=None,
        layout_name: str=None,
    ) -> None:
    r"""
    Starts the RTMP live stream for the specified conference. Once the Dolby.io Communications APIs service starts
    streaming to the target url, a `Stream.Rtmp.InProgress` Webhook event will be sent.

    See: https://docs.dolby.io/communications-apis/reference/start-rtmp

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        rtmp_url: The destination URI provided by the RTMP service.
        layout_url: Overwrites the layout URL configuration.
            This field is ignored if it is not relevant regarding recording configuration,
            for example if live_recording set to false or if the recording is MP3 only.
            - `null`: uses the layout URL configured in the dashboard (if no URL is set in the dashboard, then uses the Dolby.io default);
            - `default`: uses the Dolby.io default layout;
            - URL string: uses this layout URL
        layout_name: Defines a name for the given layout URL, which makes layout identification
            easier for customers especially when the layout URL is not explicit.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    payload = {
        'uri': rtmp_url,
    }
    add_if_not_none(payload, 'layoutUrl', layout_url)
    add_if_not_none(payload, 'layoutName', layout_name)

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/rtmp/start',
            payload=payload
        )

async def stop_rtmp(
        access_token: str,
        conference_id: str,
    ) -> None:
    r"""
    Stops the RTMP stream of the specified conference.

    See: https://docs.dolby.io/communications-apis/reference/stop-rtmp

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/rtmp/stop'
        )

async def start_rts(
        access_token: str,
        conference_id: str,
        stream_name: str,
        publishing_token: str,
        layout_url: str=None,
        layout_name: str=None,
    ) -> None:
    r"""
    Starts real-time streaming using Dolby.io Real-time Streaming services (formerly Millicast).

    See: https://docs.dolby.io/communications-apis/reference/start-rts

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.
        stream_name: The Dolby.io Real-time Streaming stream name to which the conference is broadcasted.
        publishing_token: The publishing token used to identify the broadcaster.
        layout_url: Overwrites the layout URL configuration:
            null: uses the layout URL configured in the dashboard
                (if no URL is set in the dashboard, then uses the Dolby.io default)
            default: uses the Dolby.io default layout
            URL string: uses this layout URL
        layout_name: Defines a name for the given layout URL, which makes layout identification
            easier for customers especially when the layout URL is not explicit.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    payload = {
        'stream_name': stream_name,
        'publishingToken': publishing_token,
    }
    add_if_not_none(payload, 'layoutUrl', layout_url)
    add_if_not_none(payload, 'layoutName', layout_name)

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/rts/start',
            payload=payload,
        )

async def stop_rts(
        access_token: str,
        conference_id: str,
    ) -> None:
    r"""
    Stops real-time streaming to Dolby.io Real-time Streaming services.

    See: https://docs.dolby.io/communications-apis/reference/stop-rts

    Args:
        access_token: Access token to use for authentication.
        conference_id: Identifier of the conference.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    async with CommunicationsHttpContext() as http_context:
        await http_context.requests_post(
            access_token=access_token,
            url=f'{get_comms_url_v2()}/conferences/mix/{conference_id}/rts/stop'
        )



import hashlib
import pathlib
import sys

packages_dir = pathlib.Path(sys.argv[1]).resolve().absolute()

print("Showing hash values of files to be uploaded:")

for file_object in packages_dir.iterdir():
    sha256 = hashlib.sha256()
    md5 = hashlib.md5()
    blake2_256 = hashlib.blake2b(digest_size=256 // 8)

    print(file_object)
    print("")

    content = file_object.read_bytes()

    sha256.update(content)
    md5.update(content)
    blake2_256.update(content)

    print(f"SHA256: {sha256.hexdigest()}")
    print(f"MD5: {md5.hexdigest()}")
    print(f"BLAKE2-256: {blake2_256.hexdigest()}")
    print("")



"""
dolbyio_rest_apis.core.rate_limiter
~~~~~~~~~~~~~~~

This module contains the rate limiter for the HTTP requests.
"""

import logging
import asyncio
import time

class RateLimiter:
    """
    Rate limiter for HTTP requests.
    """

    MAX_REQUESTS_PER_SECOND: int = 50

    def __init__(self):
        self._logger = logging.getLogger(RateLimiter.__name__)
        self._nb_requests_left = self.MAX_REQUESTS_PER_SECOND
        self._last_update = time.monotonic()

    async def wait_until_allowed(self):
        """
        Wait until the request is allowed to go through.
        """

        log_printed = False
        while self._nb_requests_left <= 0:
            if not log_printed:
                # Only print that message once
                log_printed = True
                self._logger.debug('This request is being throttled.')

            self._add_new_request()
            await asyncio.sleep(0.1)

        self._nb_requests_left -= 1

    def _add_new_request(self):
        now = time.monotonic()
        left = self._nb_requests_left + now - self._last_update
        if left > 0:
            self._nb_requests_left = min(left, self.MAX_REQUESTS_PER_SECOND)
            self._last_update = now

# Instance of the rate limiter to share across all APIs
RATE_LIMITER = RateLimiter()



"""
dolbyio_rest_apis.media.models.analyze_response
~~~~~~~~~~~~~~~

This module contains the Analyze models.
"""

from .job_response import JobResponse

class AnalyzeJobResult(dict):
    """The :class:`AnalyzeJobResult` object, which represents the result for an analyze job."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        if 'media_info' in dictionary:
            self.media_info = dictionary['media_info']
        if 'processed_region' in dictionary:
            self.media_info = dictionary['processed_region']

class AnalyzeJobResponse(JobResponse):
    """The :class:`AnalyzeJobResponse` object, which represents the result for an analyze job."""

    def __init__(self, job_id, dictionary: dict):
        JobResponse.__init__(self, job_id, dictionary)

        if 'result' in dictionary:
            self.result = AnalyzeJobResult(dictionary)



"""
dolbyio_rest_apis.core.http_request_error
~~~~~~~~~~~~~~~

This module contains the model HttpRequestError.
"""

from aiohttp import ClientResponse

class HttpRequestError(Exception):
    r"""HTTP exception raised from a client request."""

    def __init__(self,
            http_response: ClientResponse,
            error_type: str,
            error_code: int,
            error_reason: str,
            error_description: str):
        self.status_code = http_response.status
        self.url = http_response.url
        self.error_type = error_type
        self.error_code = error_code
        self.error_reason = error_reason
        self.error_description = error_description

        super().__init__(error_description)

    def __str__(self) -> str:
        return f'''HTTP Request Error!\r
            \tURL: {self.url}\r
            \tStatus Code: {self.status_code}\r
            \tDolby.io Error Code:  {self.error_code}
            \tDolby.io Reason:      {self.error_reason}\r
            \tDolby.io Description: {self.error_description}'''



"""
dolbyio_rest_apis.media.models.webhook
~~~~~~~~~~~~~~~

This module contains the Webhook model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default

class Webhook(dict):
    """The :class:`Webhook` object, which represents a webhook."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.webhook_id = get_value_or_default(self, 'webhook_id', None)
        self.url = None
        self.headers = None
        if 'callback' in self:
            callback = self['callback']
            self.url = get_value_or_default(callback, 'url', None)
            self.headers = get_value_or_default(callback, 'headers', None)



"""
dolbyio_rest_apis.authentication
~~~~~~~~~~~~~~~

This module contains the functions to work with the authentication API.
"""

from dolbyio_rest_apis.core.helpers import add_if_not_none
from dolbyio_rest_apis.core.urls import get_api_url
from dolbyio_rest_apis.communications.internal.http_context import CommunicationsHttpContext
from .models import AccessToken

async def get_api_token(
        app_key: str,
        app_secret: str,
        expires_in: int=None,
    ) -> AccessToken:
    r"""
    To make any API call, you must acquire a JWT (JSON Web Token) format API token.

    See: https://docs.dolby.io/communications-apis/reference/get-api-token
    
    See: https://docs.dolby.io/media-apis/reference/get-api-token

    Args:
        app_key: Your Dolby.io App Key.
        app_secret: Your Dolby.io App Secret.
        expires_in: (Optional) API token expiration time in seconds.
            The maximum value is 2,592,000, indicating 30 days.
            If no value is specified, the default is 1800, indicating 30 minutes.

    Returns:
        An :class:`AccessToken` object.

    Raises:
        HttpRequestError: If a client error one occurred.
        HTTPError: If one occurred.
    """

    data = {
        'grant_type': 'client_credentials',
    }
    add_if_not_none(data, 'expires_in', expires_in)

    async with CommunicationsHttpContext() as http_context:
        json_response = await http_context.requests_post_basic_auth(
            app_key=app_key,
            app_secret=app_secret,
            url=f'{get_api_url()}/auth/token',
            data=data
        )

    return AccessToken(json_response)



"""
Command: Remix
"""

from argparse import _SubParsersAction, Namespace
from dolbyio_rest_apis.communications import remix
import json

def command_name() -> str:
    return 'remix'

def add_arguments(sub_parsers: _SubParsersAction) -> None:
    parser = sub_parsers.add_parser(command_name(), help='Remix')

    parser.add_argument(
		'-a', '--access_token',
		help='Your API Access Token.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--cid',
		help='Identifier of the conference.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--start',
		help='Start the remix of the specified conference.',
		action='store_true',
		dest='start',
		default=False
	)

    parser.add_argument(
		'-o', '--output',
		help='Set the output format.',
		dest='output_format',
		default='json',
        choices=[ 'json', 'text' ]
	)

async def execute_command(args: Namespace) -> None:
    access_token = args.access_token
    cid = args.cid
    start = args.start
    output_format = args.output_format

    if start:
        remix_status = await remix.start(
            access_token=access_token,
            conference_id=cid
        )
    else:
        remix_status = await remix.get_status(
            access_token=access_token,
            conference_id=cid
        )

    if output_format == 'json':
        print(json.dumps(remix_status, indent=4))
    else:
        for key in remix_status.keys():
            print(f'{key}: {remix_status[key]}')



"""
dolbyio_rest_apis.media.models.transcode_response
~~~~~~~~~~~~~~~

This module contains the Transcode Response model.
"""

from dolbyio_rest_apis.core.helpers import get_value_or_default
from .job_response import JobResponse

class TranscodeJobResult(dict):
    """The :class:`TranscodeJobResult` object, which represents the result for a transcode job."""

    def __init__(self, dictionary: dict):
        dict.__init__(self, dictionary)

        self.version = get_value_or_default(dictionary, 'version', None)

class TranscodeJob(JobResponse):
    """The :class:`TranscodeJob` object, which represents the result for a transcode job."""

    def __init__(self, job_id, dictionary: dict):
        JobResponse.__init__(self, job_id, dictionary)

        if 'result' in dictionary:
            self.result = TranscodeJobResult(dictionary)



"""
Command: Streaming
"""

from argparse import _SubParsersAction, Namespace
from dolbyio_rest_apis.communications import streaming
import sys

def command_name() -> str:
    return 'streaming'

def add_arguments(sub_parsers: _SubParsersAction) -> None:
    parser = sub_parsers.add_parser(command_name(), help='Streaming')

    parser.add_argument(
		'-a', '--access_token',
		help='Your API Access Token.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--cid',
		help='Identifier of the conference.',
		required=True,
		type=str
	)

    parser.add_argument(
		'--action',
		help='Start or stop the streaming.',
		required=True,
        choices=[ 'start', 'stop' ],
		type=str
	)

    parser.add_argument(
		'--target',
		help='Perform the operation for RTMP.',
		required=True,
        choices=[ 'rtmp' ],
		type=str
	)

    parser.add_argument(
		'-u', '--url',
		help='The destination URI provided by the RTMP service.',
		nargs='*',
        required='start' in sys.argv and 'rtmp' in sys.argv,
		type=str
	)

async def execute_command(args: Namespace) -> None:
    access_token = args.access_token
    cid = args.cid
    action = args.action
    target = args.target
    url = args.url

    if target == 'rtmp':
        if action == 'start':
            print(f'Start the streaming to RTMP for the conference "{cid}".')
            print(f' - {url}')

            await streaming.start_rtmp(
                access_token=access_token,
                conference_id=cid,
                rtmp_url=url
            )
        elif action == 'stop':
            print(f'Stop the streaming to RTMP for the conference "{cid}".')

            await streaming.stop_rtmp(
                access_token=access_token,
                conference_id=cid
            )
